@@ -99,7 +99,7 @@ static int audio_buffer_size = BUFFSIZE;    /* The buffer size actually used */
 
 static int initialized=0;
 static int audio_capt;           /* Flag for capture/playback */
-static int mmap_capt;            /* Flag for mmap/read capture */
+static int mmap_io;              /* Flag for using either mmap or read,write */
 static int stereo;               /* 0: capture mono, 1: capture stereo */
 static int audio_size;           /* size of an audio sample: 8 or 16 bits */
 static int audio_rate;           /* sampling rate for audio */
@@ -124,10 +124,8 @@ char *audio_strerror(void);
 void set_timestamp(struct timeval tmstmp);
 void swpcpy(char *dst, char *src, int num);
 
-
 typedef void *(*start_routine_p)(void *);
 
-
 /* some (internally used only) error numbers */
 
 static int audio_errno = 0;
@@ -193,6 +191,8 @@ char *audio_strerror(void)
  *
  *      a_read     0: User is going to write (output) audio
  *                 1: User is going to read (input) audio
+ *      use_read_write 0: use mmap io as opposed to
+ *                     1: read/write system calls
  *      a_stereo   0: mono, 1: stereo
  *      a_size     size of an audio sample: 8 or 16 bits
  *      a_rate     sampling rate for audio
@@ -201,7 +201,8 @@ char *audio_strerror(void)
  *
  */
 
-int audio_init(int a_read, int use_read, int a_stereo, int a_size, int a_rate)
+int audio_init(int a_read, int use_read_write,
+               int a_stereo, int a_size, int a_rate)
 {
    int i;
 
@@ -213,14 +214,14 @@ int audio_init(int a_read, int use_read, int a_stereo, int a_size, int a_rate)
 
    if (a_size != 8 && a_size != 16) { audio_errno = AUDIO_ERR_ASIZE; return -1; }
 
-   if( use_read )
-	   mjpeg_info( "Using read(2) system call for capture");
+   if( use_read_write )
+	   mjpeg_info( "Using read(2)/write(2) system call for capture/playpack");
    else
-	   mjpeg_info( "Using mmap(2) system call for capture");
+	   mjpeg_info( "Using mmap(2) system call for capture/playback");
    /* Copy our parameters into static space */
 
    audio_capt = a_read;
-   mmap_capt  = !use_read;
+   mmap_io    = !use_read_write;
    stereo     = a_stereo;
    audio_size = a_size;
    audio_rate = a_rate;
@@ -321,7 +322,7 @@ int audio_init(int a_read, int use_read, int a_stereo, int a_size, int a_rate)
 	   audio_errno = AUDIO_ERR_FORK;
 	   return -1;
 	 }
-   
+
 #endif
    /* Since most probably errors happen during initialization,
       we wait until the audio task signals either success or failure */
@@ -431,7 +432,6 @@ set_timestamp(struct timeval tmstmp)
    }
 }
 
-
 /*
  * swpcpy: like memcpy, but bytes are swapped during copy
  */
@@ -633,14 +633,14 @@ int audio_write(uint8_t *buf, int size, int swap)
 #ifdef HAVE_SYS_SOUNDCARD_H
 static void system_error(const char *str, int fd, int use_strerror)
 {
-   if(use_strerror)
-      sprintf((char*)shmemptr->error_string, "Error %s - %s",str,strerror(errno));
+   if (use_strerror)
+      snprintf((char*)shmemptr->error_string, sizeof (shmemptr->error_string), "Error: %s - %s",strerror(errno),str);
    else
-      sprintf((char*)shmemptr->error_string, "Error %s",str);
+      snprintf((char*)shmemptr->error_string, sizeof (shmemptr->error_string), "Error: %s",str);
 
    shmemptr->audio_status = -1;
-   if( fd >= 0 )
-	   close(fd);
+   if (fd >= 0)
+	close(fd);
 #ifdef FORK_NOT_THREAD
       exit(1);
 #else
@@ -652,7 +652,6 @@ static void system_error(const char *str, int fd, int use_strerror)
 #ifdef HAVE_SYS_SOUNDCARD_H
 void do_audio(void)
 {
-
    int fd = -1;
    int tmp, ret, caps, afmt, frag;
    int nbdone, nbque, ndiff, nbpend, nbset, maxdiff;
@@ -670,25 +669,10 @@ void do_audio(void)
    struct sched_param schedparam;
    sigset_t blocked_signals;
 
-   /* Set the capture thread in a reasonable state - cancellation enabled
-      and asynchronous, SIGINT's ignored... */
-   /* PTHREAD_CANCEL_ASYNCHRONOUS is evil. */
-/*   if( pthread_setcancelstate( PTHREAD_CANCEL_ENABLE, NULL) )
-   {
-      system_error( "Bad pthread_setcancelstate", fd, 0 );
-   }
-   if( pthread_setcanceltype( PTHREAD_CANCEL_ASYNCHRONOUS, NULL) )
-   {
-      system_error( "Bad pthread_setcanceltype", fd, 0 );
-   }*/
-
    sigaddset( &blocked_signals, SIGINT );
-   if( pthread_sigmask( SIG_BLOCK, &blocked_signals, NULL ))
-   {
-      system_error( "Bad pthread_sigmask", fd, 0 );
-   }
+   if (pthread_sigmask( SIG_BLOCK, &blocked_signals, NULL))
+      system_error( "Bad pthread_sigmask", fd, 0);
 #endif
-	 
 
 /*
  * Fragment size and max possible number of frags
@@ -712,7 +696,9 @@ void do_audio(void)
  */
 
    audio_dev_name = getenv("LAV_AUDIO_DEV");
-   if(!audio_dev_name) audio_dev_name = "/dev/dsp";
+   if (!audio_dev_name) audio_dev_name = "/dev/dsp";
+
+   if (strlen(audio_dev_name) > 2048) audio_dev_name = "/dev/dsp";
 
    if(audio_capt)
       fd=open(audio_dev_name, O_RDONLY, 0);
@@ -752,12 +738,13 @@ void do_audio(void)
 /*
  * Check that the device has capability to do mmap and trigger
  */
-
-   ret = ioctl(fd, SNDCTL_DSP_GETCAPS, &caps);
-   if(ret<0) system_error("getting audio device capabilities",fd,1);
-
-   if (!(caps & DSP_CAP_TRIGGER) || !(caps & DSP_CAP_MMAP))
-      system_error("Soundcard cant do mmap or trigger",fd,0);
+   
+   if(mmap_io) {
+	   ret = ioctl(fd, SNDCTL_DSP_GETCAPS, &caps);
+	   if(ret<0) system_error("getting audio device capabilities",fd,1);
+	   if (!(caps & DSP_CAP_TRIGGER) || !(caps & DSP_CAP_MMAP))
+		   system_error("Soundcard cant do mmap or trigger",fd,0);
+   }
 
 /*
  * Get the size of the input/output buffer and do the mmap
@@ -796,7 +783,7 @@ void do_audio(void)
 
    tmp = info.fragstotal*info.fragsize;
 
-   if( !audio_capt || mmap_capt )
+   if( mmap_io )
    {
 	   if (audio_capt)
 		   buf=mmap(NULL, tmp, PROT_READ , MAP_SHARED, fd, 0);
@@ -804,7 +791,8 @@ void do_audio(void)
 		   buf=mmap(NULL, tmp, PROT_WRITE, MAP_SHARED, fd, 0);
 
 	   if (buf==MAP_FAILED)
-		   system_error("mapping audio buffer",fd, 1);
+		   system_error("mapping audio buffer "
+                        "(consider using read/write instead of mmap)",fd, 1);
 	   
 	   /*
 		* Put device into hold
@@ -854,14 +842,25 @@ void do_audio(void)
       for(nbque=0;nbque<info.fragstotal;nbque++)
       {
          if(!shmemptr->used_flag[NBUF(nbque)]) break;
-         memcpy(buf+nbque*info.fragsize,
-                (void*) shmemptr->audio_data[NBUF(nbque)],
-                info.fragsize);
+         if (mmap_io) {
+            memcpy(buf+nbque*info.fragsize,
+                   (void*) shmemptr->audio_data[NBUF(nbque)],
+                   info.fragsize);
+         } else {
+            write(fd,(void *)shmemptr->audio_data[NBUF(nbque)],
+                  info.fragsize);
+         }
          /* Mark the buffer as free */
          shmemptr->used_flag[NBUF(nbque)] = 0;
       }
       for(nbset=nbque;nbset<info.fragstotal;nbset++)
-         memset(buf+nbset*info.fragsize,0,info.fragsize);
+         if (mmap_io) {
+            memset(buf+nbset*info.fragsize,0,info.fragsize);
+         } else {
+            char buf[info.fragsize];
+            memset(buf,0,info.fragsize);
+            write(fd,buf,info.fragsize);
+         }
    }
 
 #ifndef FORK_NOT_THREAD
@@ -880,7 +879,7 @@ void do_audio(void)
  * Fire up audio device for mmap capture playback (not necessary for read)
  */
 
-   if( !audio_capt || mmap_capt )
+   if( mmap_io )
    {
 	   if(audio_capt)
 		   tmp = PCM_ENABLE_INPUT;
@@ -899,7 +898,7 @@ void do_audio(void)
       /* Wait until new audio data can be read/written */
 
 
-	   if( !audio_capt || mmap_capt )
+	   if( mmap_io )
 	   {
 		   FD_ZERO(&selectset);
 		   FD_SET(fd, &selectset);
@@ -919,12 +918,13 @@ void do_audio(void)
 	   }
 	   else
 	   {
-		   if( read(fd, (void *)shmemptr->audio_data[NBUF(nbdone)], info.fragsize ) 
-			   != info.fragsize )
-		   {
-			   system_error( "Sound driver returned partial fragment!\n", fd,1 );
-		   }
-
+          if (audio_capt) {
+             if( read(fd, (void *)shmemptr->audio_data[NBUF(nbdone)], info.fragsize ) 
+                     != info.fragsize )
+             {
+                system_error( "Sound driver returned partial fragment!\n", fd,1 );
+             }
+          }
 	   }
 
       /* Get time - this time is after at least one buffer has been
@@ -992,7 +992,7 @@ void do_audio(void)
 
          /* copy the ready buffers to our audio ring buffer */
 		 
-		 if( mmap_capt )
+		 if( mmap_io )
 			 nbpend = count.blocks;
 		 else
 			 nbpend = 1;
@@ -1005,7 +1005,7 @@ void do_audio(void)
             if(shmemptr->used_flag[NBUF(nbdone)])
                system_error("Audio ring buffer overflow",fd,0);
 			
-			if( mmap_capt )
+			if( mmap_io )
 				memcpy((void*) shmemptr->audio_data[NBUF(nbdone)],
 					   buf+(nbdone%info.fragstotal)*info.fragsize,
 					   info.fragsize);
@@ -1016,7 +1016,7 @@ void do_audio(void)
             ret = ioctl(fd, SNDCTL_DSP_GETIPTR, &count);
             if(ret<0) system_error("in ioctl SNDCTL_DSP_GETIPTR",fd,1);
 			
-			if( mmap_capt )
+			if( mmap_io )
 				nbpend += count.blocks;
 
             /* if nbpend >= total frags, a overrun most probably occured */
@@ -1071,9 +1071,14 @@ void do_audio(void)
             if(!shmemptr->used_flag[NBUF(nbque)]) break;
 
             if(nbque>nbdone)
-               memcpy(buf+(nbque%info.fragstotal)*info.fragsize,
-                      (void*) shmemptr->audio_data[NBUF(nbque)],
-                      info.fragsize);
+               if (mmap_io) {
+                  memcpy(buf+(nbque%info.fragstotal)*info.fragsize,
+                         (void*) shmemptr->audio_data[NBUF(nbque)],
+                         info.fragsize);
+               } else {
+                  write(fd,(void *)shmemptr->audio_data[NBUF(nbque)],
+                        info.fragsize);
+               }
 
             /* Mark the buffer as free */
             shmemptr->used_flag[NBUF(nbque)] = 0;
@@ -1083,7 +1088,13 @@ void do_audio(void)
          if(nbset<nbque) nbset = nbque;
          while(nbset-nbdone < info.fragstotal)
          {
-            memset(buf+(nbset%info.fragstotal)*info.fragsize,0,info.fragsize);
+            if (mmap_io) {
+               memset(buf+(nbset%info.fragstotal)*info.fragsize,0,info.fragsize);
+            } else {
+               char buf[info.fragsize];
+               memset(buf,0,info.fragsize);
+               write(fd,buf,info.fragsize);
+            }
             nbset++;
          }
       }
