@@ -28,18 +28,12 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <fcntl.h>
-#define COMPILE_LAV_IO_C
 #include "lav_io.h"
 
 #ifdef HAVE_LIBDV
 #include <libdv/dv.h>
 #endif
 
-#ifdef HAVE_LIBQUICKTIME
-#include <quicktime.h>
-#include <lqt.h>
-#endif
-
 extern int AVI_errno;
 
 static char video_format=' ';
@@ -222,7 +216,6 @@ static int scan_jpeg(unsigned char * jpegdata, long jpeglen, int header_only)
 
    jpeg_padded_len = p;
    return 0;
-
 }
 
 /* The query routines about the format */
@@ -237,7 +230,6 @@ int lav_query_APP_marker(char format)
       case 'A': return 0;
       case 'j': return 0;
       case 'q': return 1;
-      case 'm': return 0;
       default:  return 0;
    }
 }
@@ -252,7 +244,6 @@ int lav_query_APP_length(char format)
       case 'A': return 14;
       case 'j': return 14;
       case 'q': return 40;
-      case 'm': return 0;
       default:  return 0;
    }
 }
@@ -263,65 +254,31 @@ int lav_query_polarity(char format)
 
    switch(format)
    {
-      case 'a': return LAV_INTER_TOP_FIRST;
-      case 'A': return LAV_INTER_BOTTOM_FIRST;
-      case 'j': return LAV_INTER_TOP_FIRST;
-      case 'q': return LAV_INTER_TOP_FIRST;
-      case 'm': return LAV_INTER_TOP_FIRST;
-      default:  return LAV_INTER_TOP_FIRST;
+      case 'a': return Y4M_ILACE_TOP_FIRST;
+      case 'A': return Y4M_ILACE_BOTTOM_FIRST;
+      case 'j': return Y4M_ILACE_TOP_FIRST;
+      case 'q': return Y4M_ILACE_TOP_FIRST;
+      default:  return Y4M_ILACE_TOP_FIRST;
    }
 }
+
 lav_file_t *lav_open_output_file(char *filename, char format,
                     int width, int height, int interlaced, double fps,
                     int asize, int achans, long arate)
 {
    lav_file_t *lav_fd = (lav_file_t*) malloc(sizeof(lav_file_t));
-   char *extension, *tempfile;
+   char *tempfile;
 
    if (lav_fd == 0) { internal_error=ERROR_MALLOC; return 0; }
 
-   /* Set lav_fd */
-
    lav_fd->avi_fd      = 0;
    lav_fd->qt_fd       = 0;
    lav_fd->format      = format;
-   /* Sanity check: do not create a quicktime file that is named with .avi */
-   extension = rindex(filename, '.');
-   if (extension != NULL)
-   {
-      extension++;
-      switch(format)
-        {
-        case 'a':
-        case 'A':
-          if (strcasecmp(extension, "avi"))
-            {
-               internal_error = ERROR_FORMAT;
-               return 0;
-            }
-          break;
-        case 'q':
-          if (strcasecmp(extension,  "qt") &&
-              strcasecmp(extension, "mov") && strcasecmp(extension,"moov"))
-            {
-             internal_error = ERROR_FORMAT;
-             return 0;
-            }
-          break;
-        case 'j':
-          if (strcasecmp(extension, "jpg") && strcasecmp(extension,"jpeg"))
-            {
-              internal_error = ERROR_FORMAT;
-              return 0;
-            }
-          break;
-        }
-   }
    lav_fd->interlacing = interlaced ? lav_query_polarity(format) :
-                                      LAV_NOT_INTERLACED;
+                                      Y4M_ILACE_NONE;
    lav_fd->has_audio   = (asize>0 && achans>0);
    lav_fd->bps         = (asize*achans+7)/8;
-   lav_fd->chroma = CHROMAUNKNOWN;
+   lav_fd->chroma = Y4M_UNKNOWN;
 
    switch(format)
    {
@@ -332,7 +289,7 @@ lav_file_t *lav_open_output_file(char *filename, char format,
          lav_fd->avi_fd = AVI_open_output_file(filename);
          if(!lav_fd->avi_fd) { free(lav_fd); return 0; }
          AVI_set_video(lav_fd->avi_fd, width, height, fps, "MJPG");
-         if (asize) AVI_set_audio(lav_fd->avi_fd, achans, arate, asize, WAVE_FORMAT_PCM);
+         if (asize) AVI_set_audio(lav_fd->avi_fd, achans, arate, asize, WAVE_FORMAT_PCM, 0);
          return lav_fd;
 
       case 'j':
@@ -363,6 +320,13 @@ lav_file_t *lav_open_output_file(char *filename, char format,
          if(!lav_fd->qt_fd) { free(lav_fd); return 0; }
          quicktime_set_video(lav_fd->qt_fd, 1, width, height, fps,
                              (interlaced ? QUICKTIME_MJPA : QUICKTIME_JPEG));
+	 if (interlaced)
+	    {
+	    if (lav_fd->interlacing == Y4M_ILACE_TOP_FIRST)
+               lqt_set_fiel(lav_fd->qt_fd, 0, 2, 9);
+	    else if (lav_fd->interlacing == Y4M_ILACE_BOTTOM_FIRST)
+               lqt_set_fiel(lav_fd->qt_fd, 0, 2, 14);
+	    }
          if (asize)
 	    quicktime_set_audio(lav_fd->qt_fd, achans, arate, asize, QUICKTIME_TWOS);
          return lav_fd;
@@ -425,7 +389,7 @@ int lav_write_frame(lav_file_t *lav_file, uint8_t *buff, long size, long count)
 
    /* For interlaced video insert the apropriate APPn markers */
 
-   if(lav_file->interlacing!=LAV_NOT_INTERLACED)
+   if(lav_file->interlacing != Y4M_ILACE_NONE)
    {
       switch(lav_file->format)
       {
@@ -516,7 +480,7 @@ int lav_write_frame(lav_file_t *lav_file, uint8_t *buff, long size, long count)
          case 'a':
          case 'A':
             if(n==0)
-               res = AVI_write_frame( lav_file->avi_fd, buff, size );
+               res = AVI_write_frame( lav_file->avi_fd, buff, size, 0);
             else
                res = AVI_dup_frame( lav_file->avi_fd );
             break;
@@ -534,21 +498,17 @@ int lav_write_frame(lav_file_t *lav_file, uint8_t *buff, long size, long count)
       }
       if (res) break;
    }
-
    return res;
 }
 
 int lav_write_audio(lav_file_t *lav_file, uint8_t *buff, long samps)
 {
-   int res;
+   int res = -1;
 #ifdef HAVE_LIBQUICKTIME
    int i, j;
-   int16_t *qt_audio = (int16_t *)buff, **qt_audion;
+   int16_t *buff16 = (int16_t *)buff, **qt_audion;
    int channels = lav_audio_channels(lav_file);
-
-   qt_audion = malloc(channels * sizeof (int16_t **));
-   for (i = 0; i < channels; i++)
-	qt_audion[i] = (int16_t *)malloc(samps * lav_file->bps);
+   int bits = lav_audio_bits(lav_file);
 #endif
 
    video_format = lav_file->format; internal_error = 0; /* for error messages */
@@ -561,20 +521,40 @@ int lav_write_audio(lav_file_t *lav_file, uint8_t *buff, long samps)
          break;
 #ifdef HAVE_LIBQUICKTIME
       case 'q':
-	/* Deinterleave the audio into the two channels. */
-	for (i = 0; i < samps; i++)
-	    {
-	    for (j = 0; j < channels; j++)
-		qt_audion[j][i] = qt_audio[(channels*i) + j];
-	    }
-	res = lqt_encode_audio_track(lav_file->qt_fd, qt_audion, NULL,samps,0);
-	for (j = 0; j < channels; j++)
-	    free(qt_audion[j]);
-	free(qt_audion);
-        break;
+	if (bits != 16 || channels > 1)
+	 {
+    /* Deinterleave the audio into the two channels and/or convert
+     * bits per sample to the required format.
+     */
+    qt_audion = malloc(channels * sizeof(*qt_audion));
+    for (i = 0; i < channels; i++)
+      qt_audion[i] = malloc(samps * sizeof(**qt_audion));
+
+    if (bits == 16)
+      for (i = 0; i < samps; i++)
+			for (j = 0; j < channels; j++)
+			  qt_audion[j][i] = buff16[channels * i + j];
+   else if (bits == 8)
+		for (i = 0; i < samps; i++)
+		  for (j = 0; j < channels; j++)
+		    qt_audion[j][i] = ((int16_t)(buff[channels * i + j]) << 8) ^ 0x8000;
+
+   if (bits == 8 || bits == 16)
+      res = lqt_encode_audio_track(lav_file->qt_fd, qt_audion, NULL, samps, 0);
+
+	for (i = 0; i < channels; i++)
+		free(qt_audion[i]);
+		free(qt_audion);
+  	} 
+	else 
+	{
+		qt_audion = &buff16;
+		res = lqt_encode_audio_track(lav_file->qt_fd, qt_audion, NULL, samps, 0);
+  	}
+  break;
 #endif
       default:
-         res = -1;
+	break;
    }
 
    return res;
@@ -682,7 +662,7 @@ int lav_audio_channels(lav_file_t *lav_file)
    if(!lav_file->has_audio) return 0;
    video_format = lav_file->format; internal_error = 0; /* for error messages */
    switch(lav_file->format)
-   {
+      {
       case 'a':
       case 'A':
          return AVI_audio_channels(lav_file->avi_fd);
@@ -690,7 +670,7 @@ int lav_audio_channels(lav_file_t *lav_file)
       case 'q':
          return quicktime_track_channels(lav_file->qt_fd,0);
 #endif
-   }
+      }
    return -1;
 }
 
@@ -699,7 +679,7 @@ int lav_audio_bits(lav_file_t *lav_file)
    if(!lav_file->has_audio) return 0;
    video_format = lav_file->format; internal_error = 0; /* for error messages */
    switch(lav_file->format)
-   {
+      {
       case 'a':
       case 'A':
          return AVI_audio_bits(lav_file->avi_fd);
@@ -707,7 +687,7 @@ int lav_audio_bits(lav_file_t *lav_file)
       case 'q':
          return quicktime_audio_bits(lav_file->qt_fd,0);
 #endif
-   }
+      }
    return -1;
 }
 
@@ -795,12 +775,14 @@ int lav_set_video_position(lav_file_t *lav_file, long frame)
 
 int lav_read_frame(lav_file_t *lav_file, uint8_t *vidbuf)
 {
+int keyframe;
+
    video_format = lav_file->format; internal_error = 0; /* for error messages */
    switch(lav_file->format)
    {
       case 'a':
       case 'A':
-         return AVI_read_frame(lav_file->avi_fd,vidbuf);
+         return AVI_read_frame(lav_file->avi_fd,vidbuf, &keyframe);
 #ifdef HAVE_LIBQUICKTIME
       case 'q':
          return quicktime_read_frame(lav_file->qt_fd,vidbuf,0);
@@ -834,10 +816,6 @@ long lav_read_audio(lav_file_t *lav_file, uint8_t *audbuf, long samps)
    int16_t *qt_audio = (int16_t *)audbuf, **qt_audion;
    int channels = lav_audio_channels(lav_file);
    uint8_t b0, b1;
-
-   qt_audion = malloc(channels * sizeof (int16_t **));
-   for (i = 0; i < channels; i++)
-	qt_audion[i] = (int16_t *)malloc(samps * lav_file->bps);
 #endif
 
    if(!lav_file->has_audio)
@@ -853,6 +831,10 @@ long lav_read_audio(lav_file_t *lav_file, uint8_t *audbuf, long samps)
          return AVI_read_audio(lav_file->avi_fd,audbuf,samps*lav_file->bps)/lav_file->bps;
 #ifdef HAVE_LIBQUICKTIME
       case 'q':
+	qt_audion = malloc(channels * sizeof (int16_t **));
+	for (i = 0; i < channels; i++)
+	    qt_audion[i] = (int16_t *)malloc(samps * lav_file->bps);
+
 	start_pos = quicktime_audio_position(lav_file->qt_fd, 0);
 	lqt_decode_audio_track(lav_file->qt_fd, qt_audion, NULL, samps, 0);
 	last_pos = lqt_last_audio_position(lav_file->qt_fd, 0);
@@ -881,10 +863,9 @@ long lav_read_audio(lav_file_t *lav_file, uint8_t *audbuf, long samps)
             }
 out:
 	for (j = 0; j < channels; j++)
-            qt_audio[(channels*i) + j] = qt_audion[j][i];
-
-	 free(qt_audion);
-         return(res);
+            free(qt_audion[j]);
+	free(qt_audion);
+        return(res);
 #endif
    }
    return -1;
@@ -899,7 +880,7 @@ lav_file_t *lav_open_input_file(char *filename)
 #endif
    unsigned char *frame = NULL; /* Make sure un-init segfaults! */
    long len;
-   int jpg_height, jpg_width, ncomps, hf[3], vf[3];
+   int jpg_height, ncomps, hf[3], vf[3];
    int ierr;
 
    lav_file_t *lav_fd = (lav_file_t*) malloc(sizeof(lav_file_t));
@@ -911,12 +892,12 @@ lav_file_t *lav_open_input_file(char *filename)
    lav_fd->avi_fd      = 0;
    lav_fd->qt_fd       = 0;
    lav_fd->format      = 0;
-   lav_fd->interlacing = LAV_INTER_UNKNOWN;
+   lav_fd->interlacing = Y4M_UNKNOWN;
    lav_fd->sar_w       = 1; /* unknown - assume square pixels */
    lav_fd->sar_h       = 1; 
    lav_fd->has_audio   = 0;
    lav_fd->bps         = 0;
-   lav_fd->chroma = CHROMAUNKNOWN;
+   lav_fd->chroma = Y4M_UNKNOWN;
 
    /* open video file, try AVI first */
 
@@ -945,7 +926,7 @@ lav_file_t *lav_open_input_file(char *filename)
             strcat(errmsg, ", quicktime");
 #endif
 	    strcat(errmsg, ").\n");
-            fprintf(stderr, errmsg);
+            fprintf(stderr, "%s", errmsg);
 	    free(lav_fd);
 	    internal_error = ERROR_FORMAT; /* Format not recognized */
 	    return 0;
@@ -953,20 +934,55 @@ lav_file_t *lav_open_input_file(char *filename)
 #ifdef HAVE_LIBQUICKTIME
       else
 	{
+	quicktime_pasp_t pasp;
+	int nfields, detail;
+
 	  /* It is a quicktime file */
-	  lav_fd->qt_fd = quicktime_open(filename,1,0);
-	  video_format = 'q'; /* for error messages */
-	  if(!lav_fd->qt_fd) { free(lav_fd); return 0; }
-	  lav_fd->avi_fd = 0;
+	 lav_fd->qt_fd = quicktime_open(filename,1,0);
+	 video_format = 'q'; /* for error messages */
+	 if (!lav_fd->qt_fd)
+	    {
+	    free(lav_fd);
+	    return 0;
+	    }
+	  lav_fd->avi_fd = NULL;
 	  lav_fd->format = 'q';
 	  video_comp = quicktime_video_compressor(lav_fd->qt_fd,0);
 	  /* We want at least one video track */
-	  if(quicktime_video_tracks(lav_fd->qt_fd) < 1)
-	    {
-	      lav_close(lav_fd);
-	      internal_error = ERROR_FORMAT;
-	      return 0;
-	    }
+	  if (quicktime_video_tracks(lav_fd->qt_fd) < 1)
+	     {
+	     lav_close(lav_fd);
+	     internal_error = ERROR_FORMAT;
+	     return 0;
+	     }
+/*
+ * If the quicktime file has the sample aspect atom then use it to set
+ * the sar values in the lav_fd structure.  Hardwired (like everywhere else)
+ * to only look at track 0.
+*/
+	  if (lqt_get_pasp(lav_fd->qt_fd, 0, &pasp) != 0)
+	     {
+	     lav_fd->sar_w = pasp.hSpacing;
+	     lav_fd->sar_h = pasp.vSpacing;
+	     }
+/*
+ * If a 'fiel' atom is present (not guaranteed) then use it to set the
+ * interlacing type.
+*/
+	  if (lqt_get_fiel(lav_fd->qt_fd, 0, &nfields, &detail) != 0)
+	     {
+	     if (nfields == 2)
+	        {
+		if (detail == 14 || detail == 6)
+		   lav_fd->interlacing = Y4M_ILACE_BOTTOM_FIRST;
+		else if (detail == 9 || detail == 1)
+		   lav_fd->interlacing = Y4M_ILACE_TOP_FIRST;
+		else
+		   mjpeg_warn("unknown 'detail' in 'fiel' atom: %d", detail);
+	        }
+	     else
+	        lav_fd->interlacing = Y4M_ILACE_NONE;
+	     }
 	  /* Check for audio tracks */
 	  lav_fd->has_audio = 0;
 	  if (quicktime_audio_tracks(lav_fd->qt_fd))
@@ -974,7 +990,7 @@ lav_file_t *lav_open_input_file(char *filename)
 	     audio_comp = quicktime_audio_compressor(lav_fd->qt_fd,0);
 	     if (strncasecmp(audio_comp, QUICKTIME_TWOS,4)==0)
 		lav_fd->has_audio = 1;
-	      }
+	     }
 	 }
 #endif
    }
@@ -990,46 +1006,84 @@ lav_file_t *lav_open_input_file(char *filename)
    lav_fd->bps = (lav_audio_channels(lav_fd)*lav_audio_bits(lav_fd)+7)/8;
    if(lav_fd->bps==0) lav_fd->bps=1; /* make it save since we will divide by that value */
 
-   /* Check compressor, no further action if not Motion JPEG */
+/*
+ * Check compressor.  The YUV checks are not right (the support code appears 
+ * incorrect and/or incomplete).  In particular yuv2 is a packed format not
+ * planar and YV12 has the U and V planes reversed from IYUV (which is what
+ * the support code appears to think is YUV420).  My hunch is that folks are 
+ * only using DV and MJPG so the YUV bugs aren't being triggered. 
+ *
+ * But at least now the checks are consolidated in 1 place instead of being
+ * duplicated in two places (editlist.c and this file).
+*/
 
    ierr  = 0;
 
-   if(strncasecmp(video_comp,"mjpg",4)!=0 &&
-      strncasecmp(video_comp,"mjpa",4)!=0 &&
-      strncasecmp(video_comp,"jpeg",4)!=0 )
-   {
+   if (strncasecmp(video_comp, "yv12", 3) == 0)
+      {
+      lav_fd->dataformat = DATAFORMAT_YUV420;
+/*
+ * This is probably not correct.  But since 'yv12' isn't really supported it
+ * doesn't matter ;)
+*/
+      lav_fd->chroma = Y4M_CHROMA_420JPEG;
+      }
+   else if (strncasecmp(video_comp, "yuv2", 4) == 0)
+      {
+      lav_fd->dataformat = DATAFORMAT_YUV422;
+      lav_fd->chroma = Y4M_CHROMA_422;
+      }
+   else if (strncasecmp(video_comp, "dv", 2) == 0)
+      {
+      lav_fd->dataformat = DATAFORMAT_DV2;
+      lav_fd->interlacing = Y4M_ILACE_BOTTOM_FIRST;
+      }
+   else if (strncasecmp(video_comp, "mjp", 3) == 0 ||
+            strncasecmp(video_comp, "jpeg", 4) == 0)
+      lav_fd->dataformat = DATAFORMAT_MJPG;
+   else
+      {
+      internal_error = ERROR_FORMAT;
+      goto ERREXIT;
+      }
+
 #ifdef HAVE_LIBDV
-   if(strncasecmp(video_comp,"dvsd",4)==0
-#ifdef HAVE_LIBQUICKTIME
-      || strncasecmp(video_comp,QUICKTIME_DV,4)==0
-#endif
-      || strncasecmp(video_comp,"dv",2)==0) {
-       ierr = check_DV2_input(lav_fd);
-       if (ierr) goto ERREXIT;
-       /* DV is always interlaced, bottom first */
-       lav_fd->interlacing = LAV_INTER_BOTTOM_FIRST; 
-   }
+   if (lav_fd->dataformat == DATAFORMAT_DV2)
+      {
+      ierr = check_DV2_input(lav_fd);
+      if (ierr)
+	 goto ERREXIT;
+      }
 #endif /* HAVE_LIBDV */
 
-   if (strncasecmp(video_comp,"yuv",3)==0 || strncasecmp(video_comp, "yv12",4)==0)
-      {
-      lav_fd->chroma = CHROMA420;
-      if (ierr) goto ERREXIT;
-      }
-   return lav_fd;
-   }
+   if (lav_fd->dataformat != DATAFORMAT_MJPG)
+      return lav_fd;
 
-   /* Make some checks on the video source, we read the first frame for that */
+/*
+ * From here on down is MJPG only code - the yuv and dv cases have been handled
+ * above.  Make some checks on the video source.  Read the first frame for this.
+*/
 
    frame = NULL;
-   if ( lav_set_video_position(lav_fd,0) ) goto ERREXIT;
-   if ( (len = lav_frame_size(lav_fd,0)) <=0 ) goto ERREXIT;
-   if ( (frame = (unsigned char*) malloc(len)) == 0 ) { ierr=ERROR_MALLOC; goto ERREXIT; }
-
-   if ( lav_read_frame(lav_fd,frame) <= 0 ) goto ERREXIT;
+   if (lav_set_video_position(lav_fd,0))
+      goto ERREXIT;
+   if ((len = lav_frame_size(lav_fd,0)) <=0)
+      goto ERREXIT;
+   if ((frame = (unsigned char*) malloc(len)) == 0)
+      {
+      ierr=ERROR_MALLOC;
+      goto ERREXIT;
+      }
+   if (lav_read_frame(lav_fd,frame) <= 0)
+      goto ERREXIT;
    /* reset video position to 0 */
-   if ( lav_set_video_position(lav_fd,0) ) goto ERREXIT;
-   if( scan_jpeg(frame, len, 1) ) { ierr=ERROR_JPEG; goto ERREXIT; }
+   if (lav_set_video_position(lav_fd,0))
+      goto ERREXIT;
+   if (scan_jpeg(frame, len, 1))
+      {
+      ierr=ERROR_JPEG;
+      goto ERREXIT;
+      }
 
    /* We have to look to the JPEG SOF marker for further information
       The SOF marker has the following format:
@@ -1070,30 +1124,24 @@ lav_file_t *lav_open_input_file(char *filename)
 	  {
 		 if( vf[0] == vf[1] && vf[0] == vf[2] )
 		 {
-			 lav_fd->chroma = CHROMA422;
+			 lav_fd->chroma = Y4M_CHROMA_422;
 		 }
 		 else if( vf[0] == 2*vf[1] && vf[0] == 2*vf[2] )
-			 lav_fd->chroma = CHROMA420;
+			 lav_fd->chroma = Y4M_CHROMA_420JPEG;
 		 else		
-			 lav_fd->chroma = CHROMAUNKNOWN;
+			 lav_fd->chroma = Y4M_UNKNOWN;
 	  }
 	  else
-		  lav_fd->chroma = CHROMAUNKNOWN;
+		  lav_fd->chroma = Y4M_UNKNOWN;
    }
 
    /* Check if video is interlaced */
-
    /* height and width are encoded in the JPEG SOF marker at offsets 5 and 7 */
 
    jpg_height = get_int2(frame + jpeg_image_offset + 5);
-   jpg_width  = get_int2(frame + jpeg_image_offset + 7);
-
-   /* check height */
 
    if( jpg_height == lav_video_height(lav_fd))
-   {
-      lav_fd->interlacing = LAV_NOT_INTERLACED;
-   }
+      lav_fd->interlacing = Y4M_ILACE_NONE;
    else if ( jpg_height == lav_video_height(lav_fd)/2 )
    {
       /* Video is interlaced */
@@ -1101,7 +1149,6 @@ lav_file_t *lav_open_input_file(char *filename)
       switch(lav_fd->format)
       {
          case 'a':
-
             /* Check the APP0 Marker, if present */
 
             if(jpeg_app0_offset && 
@@ -1109,24 +1156,21 @@ lav_file_t *lav_open_input_file(char *filename)
                strncasecmp((char*)(frame + jpeg_app0_offset + 4),"AVI1",4)==0 )
             {
                 if (frame[jpeg_app0_offset+8]==1)
-                   lav_fd->interlacing = LAV_INTER_TOP_FIRST;
+                   lav_fd->interlacing = Y4M_ILACE_TOP_FIRST;
                 else
-                   lav_fd->interlacing = LAV_INTER_BOTTOM_FIRST;
+                   lav_fd->interlacing = Y4M_ILACE_BOTTOM_FIRST;
             }
             else
             {
                /* There is no default, it really depends on the
                   application which produced the AVI */
-               lav_fd->interlacing = LAV_INTER_TOP_FIRST;
+               lav_fd->interlacing = Y4M_ILACE_TOP_FIRST;
             }
-            lav_fd->format = lav_fd->interlacing == LAV_INTER_BOTTOM_FIRST ? 'A' : 'a';
+            lav_fd->format = lav_fd->interlacing == Y4M_ILACE_BOTTOM_FIRST ? 'A' : 'a';
             break;
-
          case 'q':
-            lav_fd->interlacing = LAV_INTER_TOP_FIRST;
-
-         case 'm':
-            lav_fd->interlacing = LAV_INTER_TOP_FIRST;
+            lav_fd->interlacing = Y4M_ILACE_TOP_FIRST;
+	    break;
       }
    }
    else
@@ -1258,9 +1302,8 @@ static int check_DV2_input(lav_file_t *lav_fd)
 				} 
 			break;
 		default:
-			lav_fd->sar_w = 0; /* ??? -> unknown */
-			lav_fd->sar_h = 0;
-			break;
+			ierr = ERROR_FORMAT; /* Neither 525 or 625 system */
+			goto ERREXIT;	     /* declare a format error! */
 		}
 
 	switch(decoder->sampling) {
@@ -1302,11 +1345,11 @@ static int check_DV2_input(lav_file_t *lav_fd)
 	      if (output[1][0]==0   && output[1][1]==255 &&
 		  output[2][0]==255 && output[2][1]==0) {
 		libdv_pal_yv12 = 0;
-		lav_fd->chroma = CHROMA422;
+		lav_fd->chroma = Y4M_CHROMA_422;
 		mjpeg_info("Detected libdv PAL output YUY2 (4:2:2)");
 	      } else {
 		libdv_pal_yv12 = 1;
-		lav_fd->chroma = CHROMA420;
+		lav_fd->chroma = Y4M_CHROMA_420PALDV;
 		mjpeg_info("Detected libdv PAL output YV12 (4:2:0)");
 	      }
 	      free(output[0]);
@@ -1314,25 +1357,29 @@ static int check_DV2_input(lav_file_t *lav_fd)
 	    }
 	  } else {
 	    if(libdv_pal_yv12 == 0)
-	      lav_fd->chroma = CHROMA422;
+	      lav_fd->chroma = Y4M_CHROMA_422;
 	    else
-	      lav_fd->chroma = CHROMA420;
+	      lav_fd->chroma = Y4M_CHROMA_420PALDV;
 	  }
 	  break;
 	case e_dv_sample_411:
+	   /* libdv decodes NTSC DV (native 411) as packed YUV 422 (YUY2 or 4CC 0x32595559)
+	    * where the U and V information is repeated.
+	    */
+	   lav_fd->chroma = Y4M_CHROMA_411;
+	   break;
 	case e_dv_sample_422:
-	   /* libdv decodes NTSC DV (native 411) and by default also PAL
-	    * DV (native 420) as packed YUV 422 (YUY2 or 4CC 0x32595559)
+	   /* libdv decodes PAL DV (native 420) as packed YUV 422 (YUY2 or 4CC 0x32595559)
 	    * where the U and V information is repeated.  This can be
 	    * transformed to planar 420 (YV12 or 4CC 0x32315659).
-	    * For NTSC DV this transformation is lossy.
 	    */
-	   lav_fd->chroma = CHROMA422;
+	   lav_fd->chroma = Y4M_CHROMA_422;
 	   break;
 	case e_dv_sample_none:
 	   /* FIXME */
 	   break;
 	}
+	free(frame);
 	dv_decoder_free(decoder);
 	lav_set_video_position(lav_fd,0);
 	return 0;
@@ -1363,17 +1410,16 @@ int lav_fileno(lav_file_t *lav_file)
    {
       case 'a':
       case 'A':
-         res = AVI_fileno( lav_file->avi_fd );
+         res = AVI_fileno(lav_file->avi_fd);
          break;
 #ifdef HAVE_LIBQUICKTIME
       case 'q':
-         res = fileno(((quicktime_t *)lav_file->qt_fd)->stream);
-         break;
+	res = lqt_fileno((quicktime_t *)lav_file->qt_fd);
+	break;
 #endif
       default:
          res = -1;
    }
-
    return res;
 }
 
@@ -1384,7 +1430,7 @@ uint32_t reorder_32(uint32_t todo, int big_endian)
   unsigned long reversed; 
 
   if( big_endian )
-  {
+    {
     b0 = (todo & 0x000000FF);
     b1 = (todo & 0x0000FF00) >> 8;
     b2 = (todo & 0x00FF0000) >> 16;
@@ -1392,12 +1438,8 @@ uint32_t reorder_32(uint32_t todo, int big_endian)
 
     reversed = (b0 << 24) + (b1 << 16) + (b2 << 8) +b3;
     return reversed;
-  }
-  else
-  {
-    return todo;
-  }
-
+    }
+  return todo;
 }
 
 int lav_detect_endian (void)
@@ -1415,4 +1457,3 @@ int lav_detect_endian (void)
   else
       return -1;
 }
-
