@@ -74,7 +74,7 @@ static char format=0;
 
 static int process_image_frame = -1; /* All, >-1 means not all */
 
-#define FOURCC(a,b,c,d) ( (d<<24) | ((c&0xff)<<16) | ((b&0xff)<<8) | (a&0xff) )
+#define FOURCC(a,b,c,d) ((d<<24) | ((c&0xff)<<16) | ((b&0xff)<<8) | (a&0xff))
 
 #define FOURCC_RIFF     FOURCC ('R', 'I', 'F', 'F')
 #define FOURCC_WAVE     FOURCC ('W', 'A', 'V', 'E')
@@ -83,26 +83,43 @@ static int process_image_frame = -1; /* All, >-1 means not all */
 
 static struct
 {
-   unsigned long rifftag;
-   unsigned long rifflen;
-   unsigned long wavetag;
-   unsigned long fmt_tag;
-   unsigned long fmt_len;
-      unsigned short wFormatTag;
-      unsigned short nChannels;
-      unsigned long  nSamplesPerSec;
-      unsigned long  nAvgBytesPerSec;
-      unsigned short nBlockAlign;
-      unsigned short wBitsPerSample;
-   unsigned long datatag;
-   unsigned long datalen;
+   uint32_t rifftag;
+   uint32_t rifflen;
+   uint32_t wavetag;
+   uint32_t fmt_tag;
+   uint32_t fmt_len;
+   uint16_t wFormatTag;
+   uint16_t nChannels;
+   uint32_t nSamplesPerSec;
+   uint32_t nAvgBytesPerSec;
+   uint16_t nBlockAlign;
+   uint16_t wBitsPerSample;
+   uint32_t datatag;
+   uint32_t datalen;
 } wave_hdr;
 
 static uint8_t abuff[16384];
-
+int big_endian;
 int	verbose = 1;
+
+/* Prototype definitions */
 void Usage(char *str);
 void system_error(const char *str1, const char *str2);
+void initwav_hdr(void);
+
+void initwav_hdr()
+{
+wave_hdr.rifflen = 0; /* to be filled later */
+wave_hdr.datalen = 0; /* to be filled later */
+wave_hdr.wFormatTag = 1;  /* PCM */
+
+     wave_hdr.rifftag = reorder_32(FOURCC_RIFF, big_endian);
+     wave_hdr.wavetag = reorder_32(FOURCC_WAVE, big_endian);
+     wave_hdr.fmt_tag = reorder_32(FOURCC_FMT, big_endian);
+     wave_hdr.fmt_len = reorder_32(16, big_endian);
+     wave_hdr.datatag = reorder_32(FOURCC_DATA, big_endian);
+     swab(&wave_hdr.wFormatTag, &wave_hdr.wFormatTag ,2);
+}
 
 void Usage(char *str)
 {
@@ -116,7 +133,6 @@ void Usage(char *str)
    exit(1);
 }
 
-
 void system_error(const char *str1, const char *str2)
 {
    mjpeg_error_exit1("%s: %s: %s",str1, str2, strerror(errno));
@@ -131,11 +147,10 @@ int main(int argc, char ** argv)
    char *dotptr;
    char imgfname[4096];
    long audio_bytes_out = 0;
-   int res, n, nframe;
-   int nv = 0, na = 0;
+   int res, n, nv = 0, na = 0, nframe;
    int forcestereo = 0;
 
-   while( (n=getopt(argc,argv,"o:f:i:v:")) != EOF)
+   while ((n=getopt(argc,argv,"o:f:i:v:")) != EOF)
    {
       switch(n) {
 		  
@@ -152,7 +167,7 @@ int main(int argc, char ** argv)
 		  break;
       case 'v':
 		  verbose = atoi (optarg);
-		  if( verbose < 0 || verbose >2 )
+		  if (verbose < 0 || verbose >2)
 		  {
 			  Usage (argv[0]);
 			  exit (1);
@@ -167,19 +182,24 @@ int main(int argc, char ** argv)
 
    (void)mjpeg_default_handler_verbosity(verbose);
 
-   if(outfile==0) Usage(argv[0]);
-   if(optind>=argc) Usage(argv[0]);
-   if(outfile != 0 && format == 0) {
-      if((dotptr = strrchr(outfile, '.'))) {
+   big_endian = lav_detect_endian();
+   if (big_endian < 0)
+      exit(1);
+
+   if (outfile==0) Usage(argv[0]);
+   if (optind>=argc) Usage(argv[0]);
+   if (outfile != 0 && format == 0) {
+      if ((dotptr = strrchr(outfile, '.'))) {
 #ifdef HAVE_LIBQUICKTIME
-            if(!strcasecmp(dotptr+1, "mov") || !strcasecmp(dotptr+1, "qt")
-               || !strcasecmp(dotptr+1, "moov")) format = 'q';
+            if (!strcasecmp(dotptr+1, "mov"))
+               format = 'q';
 #endif
-            if(!strcasecmp(dotptr+1, "avi")) format = 'a';
+            if (!strcasecmp(dotptr+1, "avi")) format = 'a';
+            if (!strcasecmp(dotptr+1, "wav")) format = 'w';
          }
-      if(format == '\0') format = 'a';
+      if (format == '\0') format = 'a';
    }
-   if(format!='a' && format!='q' && format!='m' && format!='i' && format!='w' && format!='W') Usage(argv[0]);
+   if (format!='a' && format!='q' && format!='i' && format!='w' && format!='A' && format!='W') Usage(argv[0]);
 
    if (process_image_frame != -1 && format!='i')
    {
@@ -192,73 +212,73 @@ int main(int argc, char ** argv)
    read_video_files(argv + optind, argc - optind, &el,0);
 
 
-   if(format == 'a' && el.video_inter == LAV_INTER_BOTTOM_FIRST) format = 'A';
+   if (format == 'a' && el.video_inter == Y4M_ILACE_BOTTOM_FIRST) format = 'A';
 
-   if((format == 'q' || format == 'm') && el.video_inter == LAV_INTER_BOTTOM_FIRST)
+   if ((format == 'q') && el.video_inter == Y4M_ILACE_BOTTOM_FIRST)
    {
       mjpeg_error_exit1("Output is Quicktime - wrong interlacing order");
    }
 
-   if(format == 'q' || format == 'a' || format == 'A' || format == 'm')
+   if (format == 'q' || format == 'a' || format == 'A')
    {
       outfd = lav_open_output_file(outfile,format,
                                    el.video_width,el.video_height,el.video_inter, 
                                    el.video_fps /* was:video_norm=='n' ? 30000.0/1001.0 : 25.0*/ ,
                                    el.audio_bits,el.audio_chans,el.audio_rate);
-      if(!outfd)
+      if (!outfd)
       {
 		  mjpeg_error_exit1("Opening output file %s: %s",
 					  outfile,lav_strerror());
       }
    }
 
-   if(format == 'w' || format == 'W')
+   if (format == 'w' || format == 'W')
    {
-      if(!el.has_audio)
+      if (!el.has_audio)
       {
          mjpeg_error_exit1("WAV output requested but no audio present");
       }
-      if( format == 'W' )
+      if (format == 'W')
       {
-          if( el.audio_bits == 16 && el.audio_chans == 1 )
+          if (el.audio_bits == 16 && el.audio_chans == 1)
 		forcestereo = 1;
           else
           {
 			  mjpeg_error_exit1("STEREOFIED WAV output request but non mono 16-bit audio present");
           }
       }
-      wave_hdr.rifftag = FOURCC_RIFF;
-      wave_hdr.rifflen = 0; /* to be filled later */
-      wave_hdr.wavetag = FOURCC_WAVE;
-      wave_hdr.fmt_tag = FOURCC_FMT;
-      wave_hdr.fmt_len = 16;
-         wave_hdr.wFormatTag      = 1;  /* PCM */
-         wave_hdr.nChannels       = forcestereo ? 2 : el.audio_chans;
-         wave_hdr.nSamplesPerSec  = el.audio_rate;
-         wave_hdr.nAvgBytesPerSec = el.audio_rate*el.audio_bps;
-         wave_hdr.nBlockAlign     = el.audio_bps;
-         wave_hdr.wBitsPerSample  = el.audio_bits;
-      wave_hdr.datatag = FOURCC_DATA;
-      wave_hdr.datalen = 0; /* to be filled later */
       
+      initwav_hdr();
+      wave_hdr.nChannels       = forcestereo ? 2 : el.audio_chans;
+      wave_hdr.nBlockAlign     = el.audio_bps;
+      wave_hdr.wBitsPerSample  = el.audio_bits;
+
+      wave_hdr.nSamplesPerSec  = reorder_32(el.audio_rate, big_endian);
+      wave_hdr.nAvgBytesPerSec = reorder_32(el.audio_rate*el.audio_bps, big_endian);
+      if (big_endian)
+	 {
+         swab(&wave_hdr.nChannels, &wave_hdr.nChannels, 2);
+         swab(&wave_hdr.nBlockAlign, &wave_hdr.nBlockAlign, 2);
+         swab(&wave_hdr.wBitsPerSample, &wave_hdr.wBitsPerSample, 2);
+	 }
+
       wavfd = fopen(outfile,"wb");
-      if(wavfd==0) system_error("opening WAV file","fopen");
+      if (wavfd==0) system_error("opening WAV file","fopen");
       res = fwrite(&wave_hdr,sizeof(wave_hdr),1,wavfd);
-      if(res!=1) system_error("writing WAV file","fwrite");
+      if (res!=1) system_error("writing WAV file","fwrite");
    }
 
    vbuff = (uint8_t*) malloc(el.max_frame_size);
-   if(vbuff==0) { mjpeg_error_exit1("malloc failed");  }
+   if (vbuff==0) { mjpeg_error_exit1("malloc failed");  }
 
    /* Quick hack by Ronald to enable one-frame picture grabbing */
    if (process_image_frame != -1)
    {
       nv = el_get_video_frame(vbuff, process_image_frame, &el);
-      sprintf(imgfname,outfile);
-      imgfd = fopen(imgfname,"wb");
-      if(imgfd==0) system_error("opening image file","fopen");
+      imgfd = fopen(outfile,"wb");
+      if (imgfd==0) system_error("opening image file","fopen");
       res = fwrite(vbuff,nv,1,imgfd);
-      if(res!=1) system_error("writing image","fwrite");
+      if (res!=1) system_error("writing image","fwrite");
       fclose(imgfd);
       mjpeg_info("Frame %d grabbed", process_image_frame);
       exit(1);
@@ -276,11 +296,10 @@ int main(int argc, char ** argv)
          case 'a':
          case 'A':
          case 'q':
-         case 'm':
             res = lav_write_frame(outfd,vbuff,nv,1);
-            if(el.has_audio && res==0)
+            if (el.has_audio && res==0)
                res = lav_write_audio(outfd,abuff,na/el.audio_bps);
-            if(res)
+            if (res)
             {
                mjpeg_error_exit1("Error writing output: %s",lav_strerror());
             }
@@ -289,23 +308,23 @@ int main(int argc, char ** argv)
          case 'i':
             sprintf(imgfname,outfile,nframe);
             imgfd = fopen(imgfname,"wb");
-            if(imgfd==0) system_error("opening image file","fopen");
+            if (imgfd==0) system_error("opening image file","fopen");
             res = fwrite(vbuff,nv,1,imgfd);
-            if(res!=1) system_error("writing image","fwrite");
+            if (res!=1) system_error("writing image","fwrite");
             fclose(imgfd);
             break;
 
          case 'w':
          case 'W':
-	    if( forcestereo  )
+	    if (forcestereo)
 	    {
 		/* Double up the samples */
 	        int i;
                 short *sbuff = (short *)abuff;
-                for( i = na-1; i >= 0; --i )
-                {
+                for (i = na-1; i >= 0; --i)
+                  {
 		  sbuff[i+i+1]=sbuff[i+i]=sbuff[i];
-                }
+                  }
 		na = na + na;
                 res = fwrite(abuff,na,1,wavfd);
  	    }
@@ -313,29 +332,29 @@ int main(int argc, char ** argv)
             {
               res = fwrite(abuff,na,1,wavfd);
             }
-            if(res!=1) system_error("writing WAV file","fwrite");
+            if (res!=1) system_error("writing WAV file","fwrite");
             audio_bytes_out += na;
             break;
       }
    }
 
-   if(format == 'q' || format == 'a' || format == 'A' || format == 'm')
+   if (format == 'q' || format == 'a' || format == 'A')
    {
       res = lav_close(outfd);
-      if(res)
+      if (res)
       {
          mjpeg_error_exit1("Closing output file: %s",lav_strerror());
       }
    }
 
-   if(format == 'w' || format == 'W')
+   if (format == 'w' || format == 'W')
    {
-      wave_hdr.rifflen = sizeof(wave_hdr) - 8 + audio_bytes_out;
-      wave_hdr.datalen = audio_bytes_out;
+     wave_hdr.rifflen = reorder_32(sizeof(wave_hdr) -8+ audio_bytes_out, big_endian);
+      wave_hdr.datalen = reorder_32(audio_bytes_out, big_endian);
       res = fseek(wavfd,0,SEEK_SET);
-      if(res) system_error("writing WAV file","fseek");
+      if (res) system_error("writing WAV file","fseek");
       res = fwrite(&wave_hdr,sizeof(wave_hdr),1,wavfd);
-      if(res!=1) system_error("writing WAV file","fwrite");
+      if (res!=1) system_error("writing WAV file","fwrite");
       fclose(wavfd);
    }
    return 0;
