@@ -35,6 +35,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
+#include <math.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -80,27 +81,26 @@
 #include <pthread.h>
 
 #ifdef HAVE_SDL
-#include <SDL/SDL.h>
+#include <SDL.h>
 #endif
 
-#include "mjpeg_types.h"
+#include "mjpeg_logging.h"
 #include "liblavplay.h"
 #include "audiolib.h"
-/*#include "lav_io.h" */
-/*#include "editlist.h" */
 #include "jpegutils.h"
 
 #ifdef	HAVE_LIBDV
 #include <libdv/dv.h>
-
-dv_decoder_t *decoder;
-int pitches[3];
-uint8_t *dv_frame[3] = {NULL,NULL,NULL};
+#endif
 
 /* Prototypes to avoid compile time warning errors */
 
 void frame_YUV422_to_YUV420P(uint8_t **, uint8_t *, int , int );
 
+#ifdef HAVE_SDL
+int frame_planar_to_packed(uint8_t *, uint8_t **, int, int, int, int, int);
+#endif
+
 /*
  * As far as I (maddog) can tell, this is what is going on with libdv-0.9
  *  and the unpacking routine... 
@@ -224,6 +224,80 @@ void frame_YUV422_to_YUV420P(uint8_t **output, uint8_t *input,
     }
 }
 
+#define FOURCC_I420 0x30323449
+#define FOURCC_I422 0x32323449
+
+#ifdef HAVE_SDL
+int frame_planar_to_packed(uint8_t *output, uint8_t **input,
+			   int width, int height, int ofmt, int ifmt, int interlaced)
+{
+  uint8_t *y, *u, *v, *sy, *su, *sv, *sye, *su0, *sv0;
+  int dy, dc, cw, scw, ch, scdv, scdh, ln, sln, px, spx;
+
+  ch = height;
+  cw = scw = width;
+#define DIV_MAX 4
+  scdv = scdh = DIV_MAX;
+  switch (ofmt) {
+  case SDL_YUY2_OVERLAY:
+    scdh /= 2;
+    cw   /= 2;
+    y = output;     dy = 2;
+    u = output + 1;
+    v = output + 3; dc = 4;
+    break;
+  case SDL_UYVY_OVERLAY:
+    scdh /= 2;
+    cw   /= 2;
+    y = output + 1; dy = 2;
+    u = output;
+    v = output + 2; dc = 4;
+    break;
+  case SDL_YVYU_OVERLAY:
+    scdh /= 2;
+    cw   /= 2;
+    y = output;     dy = 2;
+    u = output + 3;
+    v = output + 1; dc = 4;
+    break;
+  default:
+    return 1;
+  }
+  switch (ifmt) {
+  case FOURCC_I420:
+    scdv *= 2;
+  case FOURCC_I422:
+    scdh *= 2;
+    scw  /= 2;
+    sy  = input[0];
+    su0 = input[1];
+    sv0 = input[2];
+    break;
+  default:
+    return 1;
+  }
+  sye = sy + (width * height);
+
+  while (sy < sye) {
+    *y = *sy++; y += dy;
+  }
+  for (ln = 0; ln < ch; ln++) {
+    sln = ln * DIV_MAX / scdv;
+    if (interlaced) {
+      sln &= ~1;
+      sln |= (ln & 1);
+    }
+    su = su0 + (sln * scw);
+    sv = sv0 + (sln * scw);
+    for (px = 0; px < cw; px++) {
+      spx = px * DIV_MAX / scdh;
+      *u = su[spx]; u += dc;
+      *v = sv[spx]; v += dc;
+    }
+  }
+
+  return 0;
+}
 #endif
 
 
@@ -270,6 +344,13 @@ typedef struct {
    pthread_mutex_t syncinfo_mutex;
    long buffer_entry[MJPEG_MAX_BUF];
    long currently_processed_entry;
+
+   int yuvformat;
+   uint8_t *yuvbuff[3];
+# ifdef	HAVE_LIBDV
+   dv_decoder_t *decoder;
+   uint16_t pitches[3];
+# endif
 #endif
 
    int data_format[MJPEG_MAX_BUF];
@@ -293,44 +374,19 @@ typedef struct {
 static void lavplay_msg(int type, lavplay_t *info, const char format[], ...) GNUC_PRINTF(3,4);
 static void lavplay_msg(int type, lavplay_t *info, const char format[], ...)
 {
-   if (!info)
-   {
-      /* we can't let errors pass without giving notice */
-      char buf[1024];
-      va_list args;
+   char buf[1024];
+   va_list args;
 
-      va_start(args, format);
-      vsnprintf(buf, sizeof(buf)-1, format, args);
+   va_start(args, format);
+   vsnprintf(buf, sizeof(buf)-1, format, args);
+   va_end(args);
 
-      printf("**ERROR: %s\n", buf);
-
-      va_end(args);
-   }
+   if (!info) 		/* we can't let errors pass without giving notice */
+      mjpeg_error("**ERROR: %s", buf);
    else if (info->msg_callback)
-   {
-      char buf[1024];
-      va_list args;
-
-      va_start(args, format);
-      vsnprintf(buf, sizeof(buf)-1, format, args);
-
       info->msg_callback(type, buf);
-
-      va_end(args);
-   }
    else if (type == LAVPLAY_MSG_ERROR)
-   {
-      /* we can't let errors pass without giving notice */
-      char buf[1024];
-      va_list args;
-
-      va_start(args, format);
-      vsnprintf(buf, sizeof(buf)-1, format, args);
-
-      printf("**ERROR: %s\n", buf);
-
-      va_end(args);
-   }
+      mjpeg_error("**ERROR: %s", buf);
 }
 
 
@@ -465,18 +521,16 @@ static int lavplay_get_video(lavplay_t *info, uint8_t *buff, long frame_num)
  ******************************************************/
 
 static int lavplay_get_audio(lavplay_t *info, uint8_t *buff, long frame_num, int mute)
-{
+   {
+   int num_samps;
+
    if (info->get_audio_sample)
-   {
-      int num_samps;
       info->get_audio_sample(buff, &num_samps, frame_num);
-      return num_samps;
-   }
    else
-   {
-      return el_get_audio_data(buff, frame_num, info->editlist, mute);
+      num_samps =  el_get_audio_data(buff, frame_num, info->editlist, mute);
+/* fprintf(stderr, "get_audio_sample %x num_samps %x frame_num %x\n", info->get_audio_sample, num_samps, frame_num); */
+   return(num_samps);
    }
-}
 
 
 /******************************************************
@@ -664,9 +718,8 @@ static int lavplay_SDL_update(lavplay_t *info, uint8_t *jpeg_buffer,
 			      int buf_len)
 {
    video_playback_setup *settings = (video_playback_setup *)info->settings;
-# ifdef HAVE_LIBDV
    uint8_t *output[3];
-#endif
+   uint8_t *frame_tmp;
    EditList *editlist = info->editlist;
 
 
@@ -675,33 +728,50 @@ static int lavplay_SDL_update(lavplay_t *info, uint8_t *jpeg_buffer,
    /* decode frame to yuv */
    switch (data_format) {
    case DATAFORMAT_MJPG:
-   decode_jpeg_raw (jpeg_buffer, buf_len,
-      editlist->video_inter>0&&info->exchange_fields?(editlist->video_inter+1)%2+1:editlist->video_inter,
-      CHROMA420,
-      editlist->video_width, editlist->video_height, 
-      settings->yuv_overlay->pixels[0], 
-      settings->yuv_overlay->pixels[2],
-      settings->yuv_overlay->pixels[1]);
+      decode_jpeg_raw(jpeg_buffer, buf_len,
+		      editlist->video_inter>0&&info->exchange_fields?(editlist->video_inter+1)%2+1:editlist->video_inter,
+		      Y4M_CHROMA_422,
+		      editlist->video_width, editlist->video_height,
+		      settings->yuvbuff[0],
+		      settings->yuvbuff[1],
+		      settings->yuvbuff[2]);
+      frame_planar_to_packed(settings->yuv_overlay->pixels[0], settings->yuvbuff,
+			     editlist->video_width, editlist->video_height,
+			     settings->yuvformat, FOURCC_I422, 1);
       break;
    case DATAFORMAT_YUV420:
-      memcpy(settings->yuv_overlay->pixels[0],
-	     jpeg_buffer,
-	     editlist->video_width * editlist->video_height);
-      memcpy(settings->yuv_overlay->pixels[2],
-	     jpeg_buffer + (editlist->video_width * editlist->video_height),
-	     (editlist->video_width * editlist->video_height / 4));
-      memcpy(settings->yuv_overlay->pixels[1],
-	     jpeg_buffer + (editlist->video_width * editlist->video_height * 5 / 4),
-	     (editlist->video_width * editlist->video_height / 4));
+      output[0] = jpeg_buffer;
+      output[1] = jpeg_buffer + (editlist->video_width * editlist->video_height);
+      output[2] = jpeg_buffer + (editlist->video_width * editlist->video_height * 5 / 4);
+      frame_planar_to_packed(settings->yuv_overlay->pixels[0], output,
+			     editlist->video_width, editlist->video_height,
+			     settings->yuvformat, FOURCC_I420, 1);
       break;
 
 # ifdef HAVE_LIBDV
    case DATAFORMAT_DV2:
-      dv_parse_header(decoder, jpeg_buffer);
-      switch(decoder->sampling) {
-      /* FIXME: not implemented, do default: */
+      dv_parse_header(settings->decoder, jpeg_buffer);
+      switch(settings->decoder->sampling) {
       case e_dv_sample_420:
+	/* libdv decodes PAL DV directly as planar YUV 420
+	 * (YV12 or 4CC 0x32315659) if configured with the flag
+	 * --with-pal-yuv=YV12 which is not (!) the default
+	 */
+	if (libdv_pal_yv12 == 1) {
+	  settings->pitches[0] = settings->decoder->width;
+	  settings->pitches[1] = settings->decoder->width / 2;
+	  settings->pitches[2] = settings->decoder->width / 2;
+	  dv_decode_full_frame(settings->decoder, jpeg_buffer, e_dv_color_yuv,
+			       settings->yuvbuff, (int *)settings->pitches);
+	  /* swap the U and V components */
+	  frame_tmp = settings->yuvbuff[2];
+	  settings->yuvbuff[2] = settings->yuvbuff[1];
+	  settings->yuvbuff[1] = frame_tmp;
+	  frame_planar_to_packed(settings->yuv_overlay->pixels[0], settings->yuvbuff,
+				 editlist->video_width, editlist->video_height,
+				 settings->yuvformat, FOURCC_I420, 1);
 	  break;
+	}
       case e_dv_sample_411:
       case e_dv_sample_422:
          /* libdv decodes NTSC DV (native 411) and by default also PAL
@@ -710,18 +780,12 @@ static int lavplay_SDL_update(lavplay_t *info, uint8_t *jpeg_buffer,
           * transformed to planar 420 (YV12 or 4CC 0x32315659).
           * For NTSC DV this transformation is lossy.
           */
-         pitches[0] = decoder->width * 2;
-         pitches[1] = 0;
-         pitches[2] = 0;
+         settings->pitches[0] = settings->decoder->width * 2;
+         settings->pitches[1] = 0;
+         settings->pitches[2] = 0;
 
-	 dv_decode_full_frame(decoder, jpeg_buffer, e_dv_color_yuv,
-		dv_frame, pitches);
-
-	 output[0] = settings->yuv_overlay->pixels[0];
-	 output[1] = settings->yuv_overlay->pixels[2];
-	 output[2] = settings->yuv_overlay->pixels[1];
-	 frame_YUV422_to_YUV420P(output, dv_frame[0],
-		decoder->width,	decoder->height);
+	 dv_decode_full_frame(settings->decoder, jpeg_buffer, e_dv_color_yuv,
+		settings->yuv_overlay->pixels, (int *)settings->pitches);
 	 break;
       case e_dv_sample_none:
 	 /* FIXME */
@@ -787,9 +851,10 @@ static int lavplay_SDL_init(lavplay_t *info)
     *  except for ordering of Cb and Cr planes...
     * we swap those when we copy the data to the display buffer...
     */
+   /* FIXME: Is YUY2 best 422? */
    settings->yuv_overlay = SDL_CreateYUVOverlay(editlist->video_width,
 						editlist->video_height,
-						SDL_YV12_OVERLAY,
+						settings->yuvformat,
 						settings->screen);
    if (!settings->yuv_overlay)
    {
@@ -799,13 +864,17 @@ static int lavplay_SDL_init(lavplay_t *info)
    }
    lavplay_msg(LAVPLAY_MSG_INFO, info,
       "SDL YUV overlay: %s", settings->yuv_overlay->hw_overlay ? "hardware" : "software" );
-   if(settings->yuv_overlay->pitches[0] != settings->yuv_overlay->pitches[1]*2 ||
-      settings->yuv_overlay->pitches[0] != settings->yuv_overlay->pitches[2]*2 )
+#if 0
+   if(settings->yuv_overlay->pitches[0] != settings->yuv_overlay->pitches[1] ||
+      settings->yuv_overlay->pitches[0] != settings->yuv_overlay->pitches[2] )
    {
       lavplay_msg(LAVPLAY_MSG_ERROR, info,
-         "SDL returned non-YUV420 overlay!");
+         "SDL returned pitches[] = { %d, %d, %d }", settings->yuv_overlay->pitches[0], settings->yuv_overlay->pitches[1], settings->yuv_overlay->pitches[2]);
+      lavplay_msg(LAVPLAY_MSG_ERROR, info,
+         "SDL returned non-YUV422 overlay!");
       return 0;
    }
+#endif
 
    settings->jpegdims.x = 0; /* This is not going to work with interlaced pics !! */
    settings->jpegdims.y = 0;
@@ -1502,11 +1571,11 @@ static int lavplay_init(lavplay_t *info)
 
 #ifdef HAVE_V4L 
    struct video_capability vc;
+   int hn;
 #endif 
 
    video_playback_setup *settings = (video_playback_setup *)info->settings;
    EditList *editlist = info->editlist;
-   int hn;
 
    if (editlist->video_frames == 0 && !info->get_video_frame)
    {
@@ -1565,25 +1634,33 @@ static int lavplay_init(lavplay_t *info)
    /* init SDL if we want SDL */
    if (info->playback_mode == 'S')
    {
+      char *env = getenv("LAVPLAY_VIDEO_FORMAT");
+      settings->yuvformat = (env? ((env[0])|(env[1]<<8)|(env[2]<<16)|(env[3]<<24)): SDL_YUY2_OVERLAY);
       if (!info->sdl_width) info->sdl_width = editlist->video_width;
       if (!info->sdl_height) info->sdl_height = editlist->video_height;
       if (!lavplay_SDL_init(info))
          return 0;
-   }
-#endif
 
-#ifdef	HAVE_LIBDV
-   decoder = dv_decoder_new(0,0,0);
-   decoder->quality = DV_QUALITY_BEST;
-   dv_frame[0] = (uint8_t *)malloc(3 * /*param->output_width * param->output_height */ 720 * 480 * 4);
-   dv_frame[1] = NULL;
-   dv_frame[2] = NULL;
+      settings->yuvbuff[0] = (uint8_t *)malloc(editlist->video_width * editlist->video_height * 2);
+      if (!settings->yuvbuff[0])
+      {
+         lavplay_msg (LAVPLAY_MSG_ERROR, info,
+            "Malloc error, you\'re probably out of memory");
+         return 0;
+      }
+      settings->yuvbuff[1] = settings->yuvbuff[0] + (editlist->video_width * editlist->video_height);
+      settings->yuvbuff[2] = settings->yuvbuff[0] + (editlist->video_width * editlist->video_height * 3 / 2);
+# ifdef	HAVE_LIBDV
+      settings->decoder = dv_decoder_new(0,0,0);
+      settings->decoder->quality = DV_QUALITY_BEST;
+# endif
+   }
 #endif
 
 
    if (editlist->has_audio && info->audio)
    {
-      if (audio_init(0, 0, (editlist->audio_chans>1), editlist->audio_bits,
+      if (audio_init(0, info->use_write, (editlist->audio_chans>1), editlist->audio_bits,
          editlist->audio_rate)) /* increase this last argument to test sync */
       {
          lavplay_msg(LAVPLAY_MSG_ERROR, info,
@@ -1622,11 +1699,10 @@ static int lavplay_init(lavplay_t *info)
    bp.norm = (editlist->video_norm == 'n') ? VIDEO_MODE_NTSC : VIDEO_MODE_PAL;
    lavplay_msg(LAVPLAY_MSG_INFO, info,
       "Output norm: %s", bp.norm==VIDEO_MODE_NTSC?"NTSC":"PAL");
-   hn = bp.norm==VIDEO_MODE_NTSC?480:576; /* Height of norm */
-
    if (info->playback_mode != 'S')
    {
 #ifdef HAVE_V4L
+      hn = bp.norm==VIDEO_MODE_NTSC?480:576; /* Height of norm */
       /* set correct width of device for hardware
        * DC10(+): 768 (PAL/SECAM) or 640 (NTSC), Buz/LML33: 720
        */
@@ -1752,7 +1828,7 @@ static int lavplay_init(lavplay_t *info)
 #endif
    
    /* Set field polarity for interlaced video */
-   bp.odd_even = (editlist->video_inter==LAV_INTER_TOP_FIRST);
+   bp.odd_even = (editlist->video_inter == Y4M_ILACE_TOP_FIRST);
    /*if (info->exchange_fields) bp.odd_even = !bp.odd_even; */
    /*this is already done by the open_files() function */
 
@@ -1784,7 +1860,6 @@ static void lavplay_playback_cycle(lavplay_t *info)
    double tdiff1, tdiff2;
    int n;
    int first_free, frame, skipv, skipa, skipi, nvcorr;
-   long frame_number[256]; /* Must be at least as big as the number of buffers used */
 
    stats.stats_changed = 0;
    stats.num_corrs_a = 0;
@@ -1806,7 +1881,6 @@ static void lavplay_playback_cycle(lavplay_t *info)
    }
    for(n=0;n<settings->br.count;n++) /* TODO: maybe br.count-1? */
    {
-      frame_number[n] = settings->current_frame_num;
       lavplay_mjpeg_queue_buf(info, n, 1);
    }
    stats.nqueue = settings->br.count;
@@ -1903,7 +1977,6 @@ static void lavplay_playback_cycle(lavplay_t *info)
 
          /* Read one frame, break if EOF is reached */
          frame = n % settings->br.count;
-         frame_number[frame] = settings->current_frame_num;
 #ifdef HAVE_SDL
 	 settings->buffer_entry[frame] = editlist->frame_list[settings->current_frame_num];
 #endif
@@ -1979,6 +2052,7 @@ static void *lavplay_playback_thread(void *data)
    {
       SDL_FreeYUVOverlay(settings->yuv_overlay);
       SDL_Quit();
+      free(settings->yuvbuff[0]);
    }
 #endif
 
@@ -2597,7 +2671,7 @@ int lavplay_open(lavplay_t *info, char **files, int num_files)
    else if (editlist->video_width == new_eli->video_width &&
       editlist->video_height == new_eli->video_height &&
       editlist->video_inter == new_eli->video_inter &&
-      abs(editlist->video_fps - new_eli->video_fps) < 0.0000001 &&
+      fabs(editlist->video_fps - new_eli->video_fps) < 0.0000001 &&
       editlist->has_audio == new_eli->has_audio &&
       editlist->audio_rate == new_eli->audio_rate &&
       editlist->audio_chans == new_eli->audio_chans &&
@@ -2625,11 +2699,11 @@ int lavplay_open(lavplay_t *info, char **files, int num_files)
    {
       switch (info->editlist->video_inter)
       {
-         case LAV_INTER_BOTTOM_FIRST:
-            info->editlist->video_inter = LAV_INTER_TOP_FIRST;
+         case Y4M_ILACE_BOTTOM_FIRST:
+            info->editlist->video_inter = Y4M_ILACE_TOP_FIRST;
             break;
-         case LAV_INTER_TOP_FIRST:
-            info->editlist->video_inter = LAV_INTER_BOTTOM_FIRST;
+         case Y4M_ILACE_TOP_FIRST:
+            info->editlist->video_inter = Y4M_ILACE_BOTTOM_FIRST;
             break;
          default:
             lavplay_msg(LAVPLAY_MSG_WARNING, info,
