@@ -19,6 +19,11 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -31,78 +36,6 @@
 
 #include "yuv4mpeg.h"
 
-static char **parse_cmdline (char *cmdline)
-{
-  char **argv;
-  char *p = cmdline;
-  int i, argc = 0;
-  
-  if (!p) return 0;
-  if (!*p) return 0;
-  
-  while (*p) {    
-    while (!isspace(*p)) {
-      p++;
-      if (!*p) {
-	argc++;
-	goto EOL_HANDLE;
-      }
-    }
-    argc++;     
-    while (isspace(*p)) {
-      p++;
-      if (!*p) goto EOL_HANDLE;
-    }
-  }
-
- EOL_HANDLE:
-  argv = malloc((argc + 1) * sizeof(argv[0]));
-  for (p = cmdline, i = 0; i < argc; i++) {
-    argv[i] = p;
-    while (!isspace(*(++p)));
-    p[0] = '\0';
-    while (isspace(*(++p)))
-      if (!p[0]) break;
-  }
-  argv[argc] = NULL;
-  
-  return argv;
-}
-
-
-static int fork_child (char *child) {
-
-   int n;
-   int pid;
-   int mypipe[2];
-   char **myargv;
-
-   if (pipe (mypipe)) {
-      mjpeg_error( "Couldn't create pipe to %s", child);
-      exit (1);
-   }
-   if ((pid = fork ())<0) {
-      mjpeg_error( "Couldn't fork %s", child);
-      exit (1);
-   }
-   
-   if (pid) {
-      /* child redirects stdout to pipe */
-      close (mypipe[0]);
-      close(1);
-      n = dup(mypipe[1]);
-      if(n!=1) exit(1);
-      myargv = parse_cmdline (child);
-      execvp (myargv[0], myargv);
-      return -1;
-   } else {
-      /* parent */
-      close (mypipe[1]);
-   /* fcntl (mypipe[0], F_SETFL, O_NONBLOCK); */
-      return mypipe[0];
-   }
-}
-
 static void usage( char *name )
 {
 	fprintf( stderr, "Usage: %s [-v num] <input1> <input2>\n", name);
@@ -114,10 +47,10 @@ static void usage( char *name )
 
 int main (int argc, char *argv[])
 {
+  FILE *sfp0, *sfp1;
   int stream0;   /* read from input 1 */
   int stream1;   /* read from input 2 */
   int outstream = 1;  /* output to stdout */
-  int w, h;
   int fstarg;
   unsigned char *yuv0[3];
   unsigned char *yuv1[3];
@@ -135,6 +68,8 @@ int main (int argc, char *argv[])
   y4m_init_frame_info(&finfo0);
   y4m_init_frame_info(&finfo1);
   
+  y4m_accept_extensions(1);
+
   if ((argc != 3) && (argc != 5)) 
     usage(argv[0]);
   
@@ -153,22 +88,26 @@ int main (int argc, char *argv[])
   command0 = strdup(argv[fstarg]);
   command1 = strdup(argv[fstarg + 1]);
    
-  mjpeg_info("0 Trying to create pipe to '%s'", command0);
-  stream0 = fork_child(command0);
-  mjpeg_info("1 Trying to create pipe to '%s'", command1);
-  stream1 = fork_child(command1);
+  sfp0 = popen(command0, "r");
+  if (!sfp0)
+     mjpeg_error_exit1("popen(%s) failed", command0);
+  sfp1 = popen(command1, "r");
+  if (!sfp1)
+     mjpeg_error_exit1("popen(%s) failed", command1);
+  stream0 = fileno(sfp0);
+  stream1 = fileno(sfp1);
 
   if ((err = y4m_read_stream_header(stream0, &sinfo0)) != Y4M_OK)
-    mjpeg_error_exit1("Failed to read first stream header:  %s",
-		      y4m_strerr(err));
+    mjpeg_error_exit1("Failed to read first stream header:  %s errno=%d",
+		      y4m_strerr(err), errno);
   if ((err = y4m_read_stream_header(stream1, &sinfo1)) != Y4M_OK)
     mjpeg_error_exit1("Failed to read second stream header:  %s",
 		      y4m_strerr(err));
   
   mjpeg_info("First stream parameters:");
-  y4m_log_stream_info(LOG_INFO, "1> ", &sinfo0);
+  y4m_log_stream_info(mjpeg_loglev_t("info"), "1> ", &sinfo0);
   mjpeg_info("Second stream parameters:");
-  y4m_log_stream_info(LOG_INFO, "2> ", &sinfo1);
+  y4m_log_stream_info(mjpeg_loglev_t("info"), "2> ", &sinfo1);
   
   if (y4m_si_get_width(&sinfo0) != y4m_si_get_width(&sinfo1))
     mjpeg_error_exit1("Width mismatch");
@@ -180,14 +119,14 @@ int main (int argc, char *argv[])
   if (y4m_si_get_interlace(&sinfo0) != y4m_si_get_interlace(&sinfo1))
     mjpeg_error_exit1("Interlace mismatch");
   
-  w = y4m_si_get_width(&sinfo0);
-  h = y4m_si_get_height(&sinfo0);
-  
   y4m_copy_stream_info(&osinfo, &sinfo0);
   
-  yuv0[0] = malloc (w*h);   yuv1[0] = malloc (w*h);
-  yuv0[1] = malloc (w*h/4); yuv1[1] = malloc (w*h/4);
-  yuv0[2] = malloc (w*h/4); yuv1[2] = malloc (w*h/4);
+  yuv0[0] = malloc(y4m_si_get_plane_length(&sinfo0, 0));
+  yuv1[0] = malloc(y4m_si_get_plane_length(&sinfo0, 0));
+  yuv0[1] = malloc(y4m_si_get_plane_length(&sinfo0, 1));
+  yuv1[1] = malloc(y4m_si_get_plane_length(&sinfo0, 1));
+  yuv0[2] = malloc(y4m_si_get_plane_length(&sinfo0, 2));
+  yuv1[2] = malloc(y4m_si_get_plane_length(&sinfo0, 2));
   
   if ((err = y4m_write_stream_header(outstream, &sinfo0)) != Y4M_OK)
     mjpeg_error_exit1("Failed to write output header:  %s",
@@ -212,4 +151,3 @@ int main (int argc, char *argv[])
   y4m_fini_frame_info(&finfo1);
   return 0;
 }
-
