@@ -110,7 +110,7 @@ void EncoderParams::RangeChecks()
   if (mpeg1 && vbv_buffer_code>=1024)
     mjpeg_error_exit1("vbv_buffer_size must be less than 1024 (MPEG-1)");
 
-  if (video_format<0 || video_format>5)
+  if (video_format>5)
     mjpeg_error_exit1("video_format must be in range 0...5");
 
   if (color_primaries<1 || color_primaries>7 || color_primaries==3)
@@ -123,12 +123,12 @@ void EncoderParams::RangeChecks()
   if (matrix_coefficients<1 || matrix_coefficients>7 || matrix_coefficients==3)
     mjpeg_error_exit1("matrix_coefficients must be in range 1...2 or 4...7");
 
-  if (display_horizontal_size<0 || display_horizontal_size>16383)
+  if (display_horizontal_size>16383)
     mjpeg_error_exit1("display_horizontal_size must be in range 0...16383");
-  if (display_vertical_size<0 || display_vertical_size>16383)
+  if (display_vertical_size>16383)
     mjpeg_error_exit1("display_vertical_size must be in range 0...16383");
 
-  if (dc_prec<0 || dc_prec>3)
+  if (dc_prec>3)
     mjpeg_error_exit1("intra_dc_precision must be in range 0...3");
 
   for (i=0; i<M; i++)
@@ -221,13 +221,13 @@ void EncoderParams::ProfileAndLevelChecks()
     return;
   }
 
-  if (profile<HP || profile>SP)
+  if (profile<HIGH_PROFILE || profile>SIMPLE_PROFILE)
     mjpeg_error_exit1("undefined Profile");
 
-  if (profile==SNR || profile==SPAT)
-    mjpeg_error_exit1("This encoder currently generates no scalable bitstreams");
+  if (profile < MAIN_PROFILE)
+    mjpeg_error_exit1("This encoder currently generates only main or simple profile MPEG-2");
 
-  if (level<HL || level>LL || level&1)
+  if (level<HIGH_LEVEL || level>LOW_LEVEL || level&1)
     mjpeg_error_exit1("undefined Level");
 
   maxval = &maxval_tab[(level-4) >> 1];
@@ -239,7 +239,7 @@ void EncoderParams::ProfileAndLevelChecks()
 
   /* profile (syntax) constraints */
 
-  if (profile==SP && M!=1)
+  if (profile==SIMPLE_PROFILE && M!=1)
     mjpeg_error_exit1("Simple Profile does not allow B pictures");
 
 
@@ -249,10 +249,6 @@ void EncoderParams::ProfileAndLevelChecks()
 
   /* level (parameter value) constraints */
 
-  /* Table 8-8 */
-  if (frame_rate_code>5 && level>=ML)
-    mjpeg_error_exit1("Picture rate greater than permitted in specified Level");
-
   for (i=0; i<M; i++)
   {
     if (motion_data[i].forw_hor_f_code > maxval->hor_f_code)
@@ -273,29 +269,29 @@ void EncoderParams::ProfileAndLevelChecks()
 
   if (!ignore_constraints) 
     {
+      /* Table 8-8 */
+    if (frame_rate_code > 5 && level >= ML)
+       mjpeg_error_exit1("Picture rate greater than permitted in specified Level");
+
       /* Table 8-10 */
-      if (horizontal_size > maxval->hor_size)
+     if (horizontal_size > maxval->hor_size)
 	mjpeg_error_exit1("Horizontal size is greater than permitted in specified Level");
       
-      if (vertical_size > maxval->vert_size)
+     if (vertical_size > maxval->vert_size)
 	mjpeg_error_exit1("Vertical size is greater than permitted in specified Level");
       
       /* Table 8-11 */
-      if (horizontal_size*vertical_size*frame_rate > 
-	  maxval->sample_rate)
+     if (horizontal_size*vertical_size*frame_rate > maxval->sample_rate)
 	mjpeg_error_exit1("Sample rate is greater than permitted in specified Level");
-    }      
-
   /* Table 8-12 */
-  if (bit_rate> 1.0e6 * maxval->bit_rate)
-    mjpeg_error_exit1("Bit rate is greater than permitted in specified Level");
+     if (bit_rate> 1.0e6 * maxval->bit_rate)
+        mjpeg_error_exit1("Bit rate is greater than permitted in specified Level");
+    }      
 
   /* Table 8-13 */
   if (vbv_buffer_code > maxval->vbv_buffer_size)
     mjpeg_error_exit1("vbv_buffer_size exceeds High Level limit");
 }
-
-
 
 /* 
  * Local variables:
