@@ -1,4 +1,5 @@
 #include <config.h>
+#include "cpu_accel.h"
 #include "mjpeg_types.h"
 #include "mmx.h"
 
@@ -80,7 +81,6 @@ static int32_t idct_mmx_col_round[2]={
 
 #define RPT4(x) x, x, x, x
 #define SSEMAT(A,B,C,D) RPT4((A)/(B)), RPT4(((D)*(A))/(C)-(B)), RPT4((B)), RPT4((C)/(A))
-#define ALIGN_PTR(x,a) ((void *)( (((size_t)(x))+(a)-1)&(-(a))))
 #define SHUFFLEMAP(A,B,C,D) ((A)*1+(B)*4+(C)*16+(D)*64)
 
 static float idct_sse_table[64] __attribute__ ((aligned (16)))={
@@ -483,14 +483,15 @@ void idct_mmx(int16_t *block)
 
 void idct_sse(int16_t *block)
 {
-    float temp[64+3], *dst, *altemp;
+    float temp[64] __attribute__((aligned(16)));
+    float *dst;
     int i;
     int16_t *src;
 
-    altemp=ALIGN_PTR(temp,16);
+    
 
     src=block;
-    dst=altemp;
+    dst=temp;
     for( i=0; i<2; src+=32, dst+=4, i++) {
 #define MM2XMMl(mm,x0,x1)          \
         movq_r2r     ( mm,  mm6);  \
@@ -578,7 +579,7 @@ void idct_sse(int16_t *block)
     }
 
     src=block;
-    dst=altemp;
+    dst=temp;
     for( i=0; i<2; src+=4, dst+=32, i++) {
 
 #define LOADROTATEff(src, t, x0, x1, x2, x3)           \
