@@ -61,6 +61,17 @@
 #include "imageplanes.hh"
 
 
+PictureUserData::PictureUserData(uint8_t *data, int len) :
+  data( data ),
+  len( len )
+{
+}
+
+PictureUserData::~PictureUserData()
+{
+  free(data);
+}
+
 Picture::Picture( EncoderParams &_encparams, 
                   ElemStrmWriter &writer, 
                   Quantizer &_quantizer ) :
@@ -584,9 +595,9 @@ void Picture::QuantiseAndCode(RateCtl &ratectl)
                 int mb_type = cur_mb->best_me->mb_type;
 
                 /* Code mquant and update prediction if it changed in this macroblock */
-                if( cur_mb->cbp && suggested_mquant != mquant_pred )
+                if( cur_mb->cbp && cur_mb->mquant != mquant_pred )
                 {
-                    mquant_pred = suggested_mquant;
+                    mquant_pred = cur_mb->mquant;
                     mb_type |= MB_QUANT;
                 }
 
@@ -665,6 +676,7 @@ void Picture::QuantiseAndCode(RateCtl &ratectl)
  
 void Picture::PutHeaders()
 {
+    int i;
     /* Sequence header if new sequence or we're generating for a
        format like (S)VCD that mandates sequence headers every GOP to
        do fast forward, rewind etc.
@@ -686,6 +698,11 @@ void Picture::PutHeaders()
    {
       coding->PutUserData( dummy_svcd_scan_data, sizeof(dummy_svcd_scan_data) );
    }
+
+   for( i = 0; i < org_img->user_data.size(); i++ )
+   {
+       coding->PutUserData(org_img->user_data[i]->data, org_img->user_data[i]->len);
+   }
 }
 
 /* **********************************
@@ -696,7 +713,8 @@ void Picture::PutHeaders()
  
 
 void Picture::PutTrailers( int padding_needed )
-{    coding->AlignBits();
+{
+	coding->AlignBits();
     if( padding_needed > 0 )
     {
         mjpeg_debug( "Padding coded picture to size: %d extra bytes",
@@ -756,6 +774,7 @@ double Picture::IntraCodedBlocks() const
  {
     coding->ResetBuffer();
  }
+
 
 /* 
  * Local variables:
