@@ -26,23 +26,14 @@
 #include "mjpeg_logging.h"
 #include "simd.h"
 
-#if defined(HAVE_ASM_MMX) && defined(HAVE_ASM_NASM) 
+
+void predcomp_00_mmxe(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag);
+void predcomp_10_mmxe(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag);
+void predcomp_11_mmxe(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag);
+void predcomp_01_mmxe(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag);
 
 
-void predcomp_00_mmxe(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag) __asm__ ("predcomp_00_mmxe");
-void predcomp_10_mmxe(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag) __asm__ ("predcomp_10_mmxe");
-void predcomp_11_mmxe(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag) __asm__ ("predcomp_11_mmxe");
-void predcomp_01_mmxe(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag) __asm__ ("predcomp_01_mmxe");
-
-void predcomp_00_mmx(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag) __asm__ ("predcomp_00_mmx");
-void predcomp_10_mmx(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag) __asm__ ("predcomp_10_mmx");
-void predcomp_11_mmx(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag) __asm__ ("predcomp_11_mmx");
-void predcomp_01_mmx(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag) __asm__ ("predcomp_01_mmx");
-
-
-
-
-static void pred_comp_mmxe(
+void pred_comp_mmxe(
 	uint8_t *src,
 	uint8_t *dst,
 	int lx,
@@ -81,7 +72,14 @@ static void pred_comp_mmxe(
 		
 }
 
-static void pred_comp_mmx(
+
+void predcomp_00_mmx(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag);
+void predcomp_10_mmx(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag);
+void predcomp_11_mmx(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag);
+void predcomp_01_mmx(uint8_t *src,uint8_t *dst,int lx, int w, int h, int addflag);
+
+
+void pred_comp_mmx(
 	uint8_t *src,
 	uint8_t *dst,
 	int lx,
@@ -120,18 +118,17 @@ static void pred_comp_mmx(
 		
 }
 
-void init_x86_predict( )
+
+void init_x86_predict( int32_t cpucap )
 {
-	uint32_t cpucap = cpu_accel();
 	if(cpucap & ACCEL_X86_MMXEXT ) /* AMD MMX or SSE... */
 	{
 		mjpeg_info( "SETTING EXTENDED MMX for PREDICTION!");
 		ppred_comp = pred_comp_mmxe;
 	}
-    else if(cpucap & ACCEL_X86_MMX ) /* Original MMX... */
+    	else if(cpucap & ACCEL_X86_MMX ) /* Original MMX... */
 	{
 		mjpeg_info( "SETTING MMX for PREDICTION!");
 		ppred_comp = pred_comp_mmx;
 	}
 }
-#endif
