@@ -17,6 +17,7 @@
  */
 
 #include "streamstate.h"
+#include <cassert>
 #include <algorithm>
 #include "mjpeg_types.h"
 #include "mjpeg_logging.h"
@@ -43,7 +44,7 @@ void StreamState::Init(  )
     next_split_point = BITCOUNT_OFFSET + seq_split_length;
     mjpeg_debug( "Split len = %lld", seq_split_length );
 
-    frame_num = 0;
+    frame_num = 0;          // All these counters in *encoding* order !!
     s_idx = 0;
     g_idx = 0;
     b_idx = 0;
@@ -52,7 +53,53 @@ void StreamState::Init(  )
     gop_start_frame = 0;
     gop_end_seq = true;         // At start we act as after sequence split...
     GopStart(  );
-    SetEndSeq();
+    SetTempRef();
+}
+
+/*
+    Could the following GOP be closed?
+*/
+
+bool StreamState::NextGopClosed() const 
+{ 
+    return gop_end_seq || encparams.closed_GOPs || gop_start_frame + gop_length == GetNextChapter();
+}
+
+int StreamState::GetNextChapter() const
+{
+    // advance return value when gop_start_frame exceeds or equals current chapter point
+    while( !encparams.chapter_points.empty() ) {
+        int next_chapter=encparams.chapter_points.front();
+        if( next_chapter > gop_start_frame )
+            return next_chapter;
+        encparams.chapter_points.pop_front();
+    }
+    return -1;
+}
+
+bool StreamState::CanSplitHere(int offset) const
+{
+    int nc=GetNextChapter();
+
+    // Enforce GOP dimensioning.
+    if( g_idx+offset < encparams.N_min )
+        return false;
+
+    // Enforce chapter splits...
+    if( nc < 0 )
+        return true;
+    int nc_distance = nc - (frame_num+offset);
+    if( nc_distance < 0 || g_idx+offset < encparams.N_min)
+        return false;
+
+    // Check that legal GOP sizes allow the the next chapter point to
+    // be hit exactly.  If it is between x and x+1 minimum GOPs away
+    // it must be x or less maximum gops away as only then can a mixture
+    // of gops sized somewhere between minimum and maximum sizes reach it
+    // exactly.
+    // Division must occur first; rounding down is relied upon!
+    return nc_distance <= (nc_distance/encparams.N_min) * encparams.N_max;
+
 }
 
 
@@ -60,7 +107,7 @@ void StreamState::Init(  )
   Update ss to the next sequence state.
 */
 
-void StreamState::Next(  int64_t bits_after_mux )   // Estimate of how much output would have been produced
+void StreamState::Next(  uint64_t bits_after_mux )   // Estimate of how much output would have been produced
 {
     ++frame_num;    
     ++s_idx;
@@ -76,18 +123,19 @@ void StreamState::Next(  int64_t bits_after_mux )   // Estimate of how much outp
            come out right ? */
         if( bs_short != 0 && g_idx > (int)next_b_drop )
         {
-            bigrp_length = encparams.M - 1;
             if( bs_short )
                 next_b_drop += ((double)gop_length) / (double)(bs_short+1) ;
+            bigrp_length = encparams.M - 1;
         }
-        else
+        else if( !suppress_b_frames )
             bigrp_length = encparams.M;
+        else
+            bigrp_length = 1;
 
-        // Are we starting a new GOP
+        // Do we need to start next GOP?
         if( g_idx == gop_length )
         {
-            GopStart(  );
-            // Sets frame_type == I_TYPE
+            GopStart(  );            // Sets frame_type == I_TYPE
         }
         else
         {
@@ -98,71 +146,122 @@ void StreamState::Next(  int64_t bits_after_mux )   // Estimate of how much outp
     {
         frame_type = B_TYPE;
     }
-    
+
     // Figure out if a sequence split is due...
     if( (next_split_point != 0ULL && bits_after_mux > next_split_point)
-        || (FrameInSeq() != 0 && encparams.seq_end_every_gop)
+        || (s_idx != 0 && encparams.seq_end_every_gop)
         )
     {
         mjpeg_info( "Splitting sequence next GOP start" );
         next_split_point += seq_split_length;
         gop_end_seq = true;
     }
-    
-     SetEndSeq();
+    SetTempRef();
 }
 
-void StreamState::SetEndSeq()
+/*
+	Switch stream state to force an I-Frame where a P-frame would be expected.
+    Basically: start a new GOP and do the usual house-keeping.
+*/
+
+void StreamState::ForceIFrame()
 {
-   // Ensure we have read up to the frame we plan to processs / decide
-   // EOS at (need to know if this frame is at EOF - hence must be read).
-   // TODO Cropping of Temporal reference from  Picture::Set_IP_Frame
-   // should really be handled here...
-   reader.FillBufferUpto( InputFrameNum() );
-   end_seq = frame_num == (reader.NumberOfFrames()-1) || ( g_idx == gop_length-1 && gop_end_seq);
+    assert( frame_type != B_TYPE );
+    GopStart();
+    SetTempRef();
+}
+
+/*
+    Switch stream state so that no B frames are used from the current
+    frame and onwards.
+*/
+
+void StreamState::SuppressBFrames()
+{
+    assert( b_idx == 0 && encparams.M_min == 1);
+    frame_type = P_TYPE;
+    if( encparams.M_min == 1 )
+    {
+        np += nb;
+        nb = 0;
+        bigrp_length = encparams.M_min;
+        bs_short = 0;
+        suppress_b_frames = true;
+        SetTempRef();
+    }
+}
+
+
+void StreamState::SetTempRef()
+{
+    // Ensure we have read up to the input frame we might need if the next
+    // frame in decode order is an I or P.   Make sure we don't go 'of the end'
+    // once we've reached EOS
+
+    int read_ahead_frame = frame_num+encparams.M;
+    reader.FillBufferUpto( read_ahead_frame  );
+    int last_frame = reader.NumberOfFrames()-1;
+    if( frame_type == B_TYPE )
+        temp_ref =  g_idx - 1;
+    else if( g_idx == 0 && closed_gop)
+        temp_ref =   0;
+    else // I or P frame open GOP
+        temp_ref = g_idx+(bigrp_length-1);
+
+    // At end of Sequence may need to truncate bigrp!
+    if (temp_ref > (last_frame-gop_start_frame))
+        temp_ref = (last_frame-gop_start_frame);
+    
+    // DEBUG remove when validated
+    assert( frame_num + temp_ref - g_idx == gop_start_frame + temp_ref );
+    end_stream = frame_num > last_frame;
+    end_seq =  frame_num == last_frame || ( g_idx == gop_length-1 && gop_end_seq);
 }
 
 
 void StreamState::GopStart(  )
 {
-    //int nb, np;
-    uint64_t bits_after_mux;
-    double frame_periods;
     /* If   we're starting a GOP and have gone past the current
        sequence splitting point split the sequence and
        set the next splitting point.
     */
-            
+
+    suppress_b_frames = false;
     g_idx = 0;
     b_idx = 0;
     frame_type = I_TYPE;
-    /* Sequence ended at end previous GOP so this one starts a new sequence */
-    if( gop_end_seq )
-    {
-          /* We split sequence last frame.This is the input stream display 
-           * order sequence number of the frame that will become frame 0 in display
-           * order in  the new sequence 
-           */
-          seq_start_frame += s_idx;
-          s_idx = 0;
-          gop_end_seq = false;
-          new_seq = true;
-    }
-    
 
     /* Normally set closed_GOP in first GOP only...   */
 
-    closed_gop = s_idx == 0 || encparams.closed_GOPs;
-    gop_start_frame = seq_start_frame + s_idx;
+    closed_gop = NextGopClosed(); // must call this before the gop_end_seq code
+    gop_start_frame = frame_num;
+
+    /* Sequence ended at end previous GOP so this one starts a new sequence */
+    if( gop_end_seq )
+    {
+        /* We split sequence last frame.This is the input stream display 
+         * order sequence number of the frame that will become frame 0 in display
+         * order in  the new sequence 
+         */
+        seq_start_frame =frame_num;
+        s_idx = 0;
+        gop_end_seq = false;
+        new_seq = true;
+    }
     
 
     // 
     // GOPs initially always start out maximum length - short GOPs occur when we notice
     // a P frame occurring after the minimum GOP lengthhas been reached
     //
+    // also shorten a GOP if we have an upcoming chapter point we are aiming for
+    //
 
-    gop_length = encparams.N_max;
-    mjpeg_info( "NEW GOP INIT length %d", gop_length );    
+    for( gop_length = encparams.N_max; gop_length > encparams.N_min; gop_length-- )
+        if( CanSplitHere(gop_length) )
+            break;
+
+    mjpeg_info( "NEW GOP INIT length %d", gop_length );
     /* First figure out how many B frames we're short from
        being able to achieve an even M-1 B's per I/P frame.
        
