@@ -31,9 +31,6 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
-#ifndef _WIN32
-#include <sys/param.h>
-#endif
 #include <assert.h>
 #include "mjpeg_logging.h"
 #include "bits.hpp"
@@ -60,7 +57,7 @@ BitStreamBuffering::BitStreamBuffering() :
 void BitStreamBuffering::Release()
 {
     if( bfr != 0)
-        delete bfr;
+        delete [] bfr;
     bfr = 0;
     bfr_size = 0;
     buffered = 0;
@@ -78,6 +75,7 @@ void BitStreamBuffering::Empty()
     buffered = 0;
 }
 
+
 /** sets the internal buffer size. 
     @param new_buf_size requests new internal buffer size 
 */
@@ -155,7 +153,6 @@ bool IBitStream::ReadIntoBuffer(unsigned int to_read)
 
 
 
-#define masks(idx) (1<<(idx))
 
 /*read 1 bit from the bit stream 
 @returns the read bit, 0 on EOF */
@@ -165,10 +162,9 @@ uint32_t IBitStream::Get1Bit()
 
 	if (eobs)
 		return 0;
-
-	bit = (bfr[byteidx] & masks(bitidx - 1)) >> (bitidx - 1);
+	bit = (bfr[byteidx] >> (--bitidx))& 1;
 	bitreadpos++;
-	bitidx--;
+	
 	if (!bitidx)
 	{
 		bitidx = 8;
@@ -214,10 +210,9 @@ uint32_t IBitStream::GetBits(int N)
 		{
 			if (eobs)
 				return 0;
-
-			j = (bfr[byteidx] & masks(bitidx - 1)) >> (bitidx - 1);
+			
+			j = (bfr[byteidx] >> (--bitidx))& 1;
 			bitreadpos++;
-			bitidx--;
 			if (!bitidx)
 			{
 				bitidx = 8;
@@ -282,7 +277,6 @@ void IBitStream::SeekFwdBits( unsigned int bytes_to_seek_fwd)
 { 
     assert(bitidx == 8);
     unsigned int req_byteidx = byteidx + bytes_to_seek_fwd;
-    unsigned int org_buffered = buffered;
     while( req_byteidx >= buffered && !eobs)
     {
         ReadIntoBuffer( req_byteidx - (buffered-1) );
@@ -290,7 +284,7 @@ void IBitStream::SeekFwdBits( unsigned int bytes_to_seek_fwd)
     
     eobs = ( req_byteidx >= buffered );
     if( eobs )
-        bitreadpos += (buffered-org_buffered)*8;
+        bitreadpos += (buffered - byteidx)*8;
     else
         bitreadpos += bytes_to_seek_fwd*8;
     byteidx = req_byteidx;
@@ -358,7 +352,7 @@ unsigned int IBitStream::GetBytes(uint8_t *dst, unsigned int length)
 
 	if(  bytereadpos+length > bfr_start+buffered )
 	{
-		if( !EndOfStream() )
+		if( !EndOfStream() && !scandone )
         {
 			mjpeg_error("INTERNAL ERROR: access to input stream buffer beyond last buffered byte @POS=%lld END=%d REQ=%lld + %d bytes", 
                         bytereadpos,
@@ -380,50 +374,17 @@ unsigned int IBitStream::GetBytes(uint8_t *dst, unsigned int length)
 	return to_read;
 }
 
-
-/********
+/*****
  *
- * Input stream from a standard File I/O data-source
+ * Bitstream reading is complete...
  *
- ********/
+ * Currently mainly there to allow Bug detection.  Doesn't actually *do* anything!
+ *
+ *****/
 
-IFileBitStream::IFileBitStream( const char *bs_filename,
-                                unsigned int buf_size) :
-    IBitStream()
+void IBitStream::ScanDone()
 {
-	if ((fileh = fopen(bs_filename, "rb")) == NULL)
-	{
-		mjpeg_error_exit1( "Unable to open file %s for reading.", bs_filename);
-	}
-	filename = strcpy( new char[strlen(bs_filename)+1], bs_filename );
-    streamname = filename;
-
-    SetBufSize(buf_size);
-	eobs = false;
-    byteidx = 0;
-	if (!ReadIntoBuffer())
-	{
-		if (buffered==0)
-		{
-			mjpeg_error_exit1( "Unable to read from %s.", bs_filename);
-		}
-	}
-}
-
-
-/**
-   Destructor: close the device containing the bit stream after a read
-   process
-*/
-IFileBitStream::~IFileBitStream()
-{
-	if (fileh)
-	{
-		fclose(fileh);
-		delete filename;
-	}
-	fileh = 0;
-    Release();
+    scandone = true;
 }
 
 
