@@ -22,7 +22,6 @@
 #ifndef __INTERACT_HH__
 #define __INTERACT_HH__
 
-#include <config.h>
 #ifndef _WIN32
 #include <unistd.h>
 #endif
@@ -42,26 +41,43 @@ using std::vector;
  *
  *************************************************************************/
 
+struct Workarounds
+{
+  Workarounds();
+};
+
 class MultiplexParams
 {
 public:
   unsigned int data_rate;
   unsigned int packets_per_pack;
-  int video_offset;
-  int audio_offset;
+  int video_offset;             // A/V sync offset. Always one 0 and the
+                                // other positive. Specified in 
+  int audio_offset;             // MPEG-2 CLOCKS: 1/(90000*300)-th sec
+
   unsigned int sector_size;
-  bool VBR;
+  bool VBR;                     // Force VBR even if profile suggests CBR
+  bool CBR;                     // Force CBR even if profile suggests VBR
   int mpeg;
   int mux_format;
   bool multifile_segment;
   bool always_system_headers;
   unsigned int max_PTS;
-  bool emul_vcdmplex;
   bool stills;
   int verbose;
   int max_timeouts;
-  char *outfile_pattern;
+  const char *outfile_pattern;
+  const char *vdr_index_pathname;
   int max_segment_size;
+  int min_pes_header_len;
+  int run_in_frames;            // Run-in expressed in Frame intervals
+  Workarounds workarounds;      // Special work-around flags that
+                                // constrain the syntax to suit
+                                // the foibles of particular MPEG
+                                // parsers that are (guessed) to be
+                                // actually slightly broken.  Always
+                                // off by default...
+
 };
 
 /***********************************************************************
@@ -77,11 +93,8 @@ enum StreamKind
     AC3_AUDIO,
     LPCM_AUDIO,
     DTS_AUDIO,
-    MPEG_VIDEO
-#ifdef ZALPHA
-    ,
-    Z_ALPHA
-#endif
+    MPEG_VIDEO,
+    SUBP_STREAM,
   };
 
 class JobStream
@@ -104,33 +117,21 @@ class MultiplexJob : public MultiplexParams
 public:
   MultiplexJob();
   virtual ~MultiplexJob();
-  virtual void SetFromCmdLine( unsigned int argc, char *argv[]);
-  virtual PS_Stream *GetOutputStream(
-			   unsigned mpeg,
-               unsigned int sector_size,
-               const char *filename_pat,
-               off_t max_segment_size // 0 = No Limit
-			   );
   unsigned int NumberOfTracks( StreamKind kind );
-  void GetJobStreams( vector<JobStream *> &streams, StreamKind kind );
+  void GetInputStreams( vector<JobStream *> &streams, StreamKind kind );
 
+  void SetupInputStreams( vector<IBitStream *> &inputs );
 protected:
-  virtual void InputStreamsFromCmdLine (unsigned int argc, char* argv[] );
-  virtual void Usage(char *program_name);
-  virtual bool ParseVideoOpt( const char *optarg );
-  virtual bool ParseLpcmOpt( const char *optarg );
 
 public:  
   vector<JobStream *> streams;
   vector<LpcmParams *> lpcm_param;
   vector<VideoParams *> video_param;
+  vector<SubtitleStreamParams*> subtitle_params;
   unsigned int audio_tracks;
   unsigned int video_tracks;
+  unsigned int subtitle_tracks;
   unsigned int lpcm_tracks;
-#ifdef ZALPHA
-  unsigned int z_alpha_tracks;
-#endif
-
 };
 
 
@@ -138,8 +139,8 @@ public:
     Program ID
 *************************************************************************/
  
-#define MPLEX_VER    "2.2.3"
-#define MPLEX_DATE   "$Date: 2003/11/26 04:19:33 $"
+#define MPLEX_VER    "2.2.7"
+#define MPLEX_DATE   "$Date: 2012-11-17 01:55:16 $"
 
 #endif // __INTERACT_H__
 
