@@ -124,8 +124,6 @@ void MPAStream::Init ( const int stream_num )
                 bs.StreamName()
                 );
 
-	InitAUbuffer();
-	
 	/* A.Stevens 2000 - update to be compatible up to  MPEG2.5
 	 */
     AU_start = bs.bitcount();
@@ -150,13 +148,17 @@ void MPAStream::Init ( const int stream_num )
 			mpa_slots[layer] *1000 /
 			mpa_freq_table[version_id][frequency];
 
-		size_frames[0] = framesize;
-		size_frames[1] = framesize+( layer == 0 ? 4 : 1);
+		size_frames[0] = framesize * ( layer == 0 ? 4 : 1);
+		size_frames[1] = (framesize+1) * ( layer == 0 ? 4 : 1);
 		num_frames[padding_bit]++;
         access_unit.start  = AU_start;
 		access_unit.length = size_frames[padding_bit];
 	  
 		samples_per_second = mpa_freq_table[version_id][frequency];
+		if (!samples_per_second) {
+			mjpeg_error ( "Invalid frequency in MPEG Audio stream header.");
+			exit(1);
+		}
 
 		/* Presentation time-stamping  */
 		access_unit.PTS = static_cast<clockticks>(decoding_order) * 
@@ -165,7 +167,7 @@ void MPAStream::Init ( const int stream_num )
 		access_unit.DTS = access_unit.PTS;
 		access_unit.dorder = decoding_order;
 		++decoding_order;
-		aunits.append( access_unit );
+		aunits.Append( access_unit );
 
     } else
     {
@@ -185,9 +187,9 @@ unsigned int MPAStream::NominalBitRate()
 
 unsigned int MPAStream::SizeFrame( int rate_code, int padding )
 {
-	return mpa_bitrates_kbps[version_id][layer][rate_code]  * 
+	return ( mpa_bitrates_kbps[version_id][layer][rate_code]  * 
 		mpa_slots [layer] *1000 /
-		mpa_freq_table[version_id][frequency] + padding;
+		mpa_freq_table[version_id][frequency] + padding ) * ( layer == 0 ? 4 : 1);
 }
 
 void MPAStream::FillAUbuffer(unsigned int frames_to_buffer )
@@ -215,7 +217,7 @@ void MPAStream::FillAUbuffer(unsigned int frames_to_buffer )
             mjpeg_warn("Discarding incomplete final frame MPEG audio stream %02x!",
                        stream_id
                        );
-            aunits.droplast();
+            aunits.DropLast();
             --decoding_order;
             break;
         }
@@ -247,7 +249,7 @@ void MPAStream::FillAUbuffer(unsigned int frames_to_buffer )
 		access_unit.DTS = access_unit.PTS;
 		access_unit.dorder = decoding_order;
 		decoding_order++;
-		aunits.append( access_unit );
+		aunits.Append( access_unit );
 		num_frames[padding_bit]++;
 
 		bs.GetBits( 9);
