@@ -32,9 +32,6 @@
 #include "videostrm.hpp"
 #include "stillsstream.hpp"
 #include "audiostrm.hpp"
-#ifdef ZALPHA
-#include "zalphastrm.hpp"
-#endif
 #include "multiplexor.hpp"
 
 
@@ -325,7 +322,7 @@ void Multiplexor::InitInputStreamsForStills(MultiplexJob & job )
     switch( job.mux_format )
     {
     case MPEG_FORMAT_VCD_STILL :
-        mjpeg_info( "Multiplexing VCD stills: %d stills streams.", video_strms.size() );
+        mjpeg_info( "Multiplexing VCD stills: %lu stills streams.", video_strms.size() );
         {
             frame_interval = 30; // 30 Frame periods
             if( mpa_strms.size() > 0 && video_strms.size() > 2  )
@@ -355,7 +352,7 @@ void Multiplexor::InitInputStreamsForStills(MultiplexJob & job )
         }
         break;
     case MPEG_FORMAT_SVCD_STILL :
-        mjpeg_info( "Multiplexing SVCD stills: %d stills streams %d audio streams", video_strms.size(), mpa_strms.size() );
+        mjpeg_info( "Multiplexing SVCD stills: %lu stills streams %lu audio streams", video_strms.size(), mpa_strms.size() );
         frame_interval = 30;
         if( video_strms.size() > 1 )
         {
@@ -393,10 +390,10 @@ void Multiplexor::InitInputStreamsForVideo(MultiplexJob & job )
 
     unsigned int audio_track = 0;
     unsigned int video_track = 0;
+    unsigned int subp_track = 0;
 	std::vector<VideoParams *>::iterator vidparm = job.video_param.begin();
 	std::vector<LpcmParams *>::iterator lpcmparm = job.lpcm_param.begin();
-
-
+	std::vector<SubtitleStreamParams *>::iterator subpparm = job.subtitle_params.begin();
     std::vector<JobStream *>::iterator i;
     for( i = job.streams.begin() ; i < job.streams.end() ; ++i )
     {
@@ -461,17 +458,17 @@ void Multiplexor::InitInputStreamsForVideo(MultiplexJob & job )
             ++audio_track;
         }
         break;
-#ifdef ZALPHA
-        // just copies the video parameters from the first video stream
-        case Z_ALPHA :
+        case SUBP_STREAM :
         {
-            ZAlphaStream *zalphaStrm = new ZAlphaStream( *(*i)->bs, *(job.video_param.begin()), *this);
-            zalphaStrm->Init (0);
-            estreams.push_back(zalphaStrm);
-            vstreams.push_back(zalphaStrm);
-            //++vidparm;
+            // we use audios stream as base class
+            SUBPStream *subpStrm =  new SUBPStream( *(*i)->bs, *subpparm,*this);
+            subpStrm ->Init ( subp_track );
+            estreams.push_back(subpStrm );
+            astreams.push_back(subpStrm );
+            ++subpparm;
+            ++subp_track;
         }
-#endif		
+        break;
         }
     }
 }
@@ -689,7 +686,8 @@ void Multiplexor::Init()
 	{
 		switch( (*str)->Kind() )
 		{
-		case ElementaryStream::audio :
+		 case ElementaryStream::audio :
+		 case ElementaryStream::subtitle :
 			(*str)->SetMaxPacketData( 
 				psstrm->PacketPayload( **str, NULL, NULL, 
 									   false, true, false ) 
@@ -1074,7 +1072,7 @@ void Multiplexor::Multiplex()
 	std::vector<bool> completed;
 	std::vector<bool>::iterator pcomp;
 	std::vector<ElementaryStream *>::iterator str;
-
+	
 	unsigned int packets_left_in_pack = 0; /* Suppress warning */
 	bool padding_packet;
 	bool video_first = true;
