@@ -23,7 +23,7 @@
 
 #include "altivec_motion.h"
 #include "vectorize.h"
-#include "../fastintfns.h"
+#include <math.h>
 #include "../mjpeg_logging.h"
 
 /* #define AMBER_ENABLE */
@@ -139,24 +139,19 @@ int build_sub22_mests_altivec(BUILD_SUB22_MESTS_PDECL)
     int min_weight;
 #endif
 
-#ifdef ALTIVEC_VERIFY /* {{{ */
-  if (((unsigned long)s22blk & 0x7) != 0)
-   mjpeg_error_exit1("build_sub22_mests: s22blk %% 8 != 0, (0x%X)", s22blk);
-
+#ifdef ALTIVEC_VERIFY
   if (NOT_VECTOR_ALIGNED(rowstride))
     mjpeg_error_exit1("build_sub22_mests: rowstride %% 16 != 0, (%d)",
 		rowstride);
+  if (NOT_VECTOR_ALIGNED(cres))
+    mjpeg_warn("build_sub22_mests: cres %% 16 != 0, (0x%X)",cres);
+  if (((unsigned long)s22blk & 0x7) != 0)
+   mjpeg_error_exit1("build_sub22_mests: s22blk %% 8 != 0, (0x%X)", s22blk);
+#endif 
 
   if (h != 4 && h != 8)
     mjpeg_error_exit1("build_sub22_mests: h != [4|8], (%d)", h);
 
-#if 0
-  if (NOT_VECTOR_ALIGNED(cres))
-    mjpeg_warn("build_sub22_mests: cres %% 16 != 0, (0x%X)",cres);
-#endif
-
-#endif /* }}} */
-
     AMBER_START;
 
     len = sub44set->len;
@@ -201,9 +196,9 @@ int build_sub22_mests_altivec(BUILD_SUB22_MESTS_PDECL)
     threshold = 6 * null_ctl_sad / (reduction << 2);
     vio.init.threshold = threshold;
     xy22 = (vector signed char)VCONST(0,0,0,0, 0,0,2,0, 0,0,0,2, 0,0,2,2);
-    vu32(xint) = vec_splat_u32(0xf);
+    xint = vu8(vec_splat_u32(0xf));
     xint = vec_add(xint, lvsl);
-    vu32(yint) = vec_splat_u32(1);
+    yint = vu8(vec_splat_u32(1));
     yint = vec_add(yint, xint);
 
     perm2 = vec_lvsl(0, s22blk);
@@ -218,8 +213,8 @@ int build_sub22_mests_altivec(BUILD_SUB22_MESTS_PDECL)
     ih = (h >> 1) - 1;
 
     vthreshold = vec_ld(0, (unsigned int*) &vio.init);
-    vu32(xylim) = vec_splat(vu32(vthreshold), 0);      /* vio.init.xylim */
-    vu32(vthreshold) = vec_splat(vu32(vthreshold), 1); /* vio.init.threshold */
+    xylim = vs8(vec_splat(vu32(vthreshold), 0));      /* vio.init.xylim */
+    vthreshold = vu32(vec_splat(vu32(vthreshold), 1)); /* vio.init.threshold */
 
     do { /* while (--len) */
 
@@ -463,16 +458,16 @@ int build_sub22_mests_altivec(BUILD_SUB22_MESTS_PDECL)
 	    sad22 = vec_sum4s(dif, sad22);
 
 	    /* calculate final sums {{{ */
-	    vs32(sads) = vec_sums(vs32(sads), vs32(zero));
-	    vs32(sad20) = vec_sums(vs32(sad20), vs32(zero));
-	    vs32(sad02) = vec_sums(vs32(sad02), vs32(zero));
-	    vs32(sad22) = vec_sums(vs32(sad22), vs32(zero));
+	    sads = vu32(vec_sums(vs32(sads), vs32(zero)));
+	    sad20 = vu32(vec_sums(vs32(sad20), vs32(zero)));
+	    sad02 = vu32(vec_sums(vs32(sad02), vs32(zero)));
+	    sad22 = vu32(vec_sums(vs32(sad22), vs32(zero)));
 	    /* }}} */
 
 	    /* sads = {sads, sad20, sad02, sad22} {{{ */
-	    vu32(sads) = vec_mergel(vu32(sads), vu32(sad02));
-	    vu32(sad20) = vec_mergel(vu32(sad20), vu32(sad22));
-	    vu32(sads) = vec_mergel(vu32(sads), vu32(sad20));
+	    sads = vu32(vec_mergel(vu32(sads), vu32(sad02)));
+	    sad20 = vu32(vec_mergel(vu32(sad20), vu32(sad22)));
+	    sads = vu32(vec_mergel(vu32(sads), vu32(sad20)));
 	    /* }}} */
 	} /* }}} */
 
@@ -486,8 +481,8 @@ int build_sub22_mests_altivec(BUILD_SUB22_MESTS_PDECL)
 	    vector signed char xy;
 
 	    xy = vec_ld(0, (signed char*) &vio.xy);
-	    vu32(xy) = vec_splat(vu32(xy), 0); /* splat vio.xy */
-	    xy = vec_add(xy, xy22); /* adjust xy values for elements 1-3 */
+	    xy = vs8(vec_splat(vu32(xy), 0)); /* splat vio.xy */
+	    xy = vs8(vec_add(xy, xy22)); /* adjust xy values for elements 1-3 */
 
 	    /* add distance penalty {{{ */
 	    /* penalty = (max(abs(x),abs(y))<<3) */
@@ -505,8 +500,8 @@ int build_sub22_mests_altivec(BUILD_SUB22_MESTS_PDECL)
 		 * (0,0,0,x, 0,0,0,x, 0,0,0,x, 0,0,0,x) |lvsl+(0x0000000F,...)| 
 		 * (0,0,0,y, 0,0,0,y, 0,0,0,y, 0,0,0,y) |lvsl+(0x00000010,...)|
 		 */
-		vs8(xxxx) = vec_perm(vs8(zero), xyabs, xint);
-		vs8(yyyy) = vec_perm(vs8(zero), xyabs, yint);
+		xxxx = vu32(vec_perm(vs8(zero), xyabs, xint));
+		yyyy = vu32(vec_perm(vs8(zero), xyabs, yint));
 
 		/* penalty = max(abs(x),abs(y)) << 3  */
 		xymax = vec_max(xxxx, yyyy);
@@ -536,7 +531,7 @@ int build_sub22_mests_altivec(BUILD_SUB22_MESTS_PDECL)
 	    {
 		vector bool int xymask;
 
-		vb8(xymask) = vec_cmpgt(xy, xylim);
+		xymask = vb32(vec_cmpgt(xy, xylim));
 		xymask = vec_cmpgt(vu32(xymask), zero);
 
 		/* add (saturated) xymask to sads thereby forcing
@@ -558,9 +553,9 @@ int build_sub22_mests_altivec(BUILD_SUB22_MESTS_PDECL)
 		 *
 		 * ( sad,  xy, sad,  xy, sad,  xy, sad,  xy )
 		 */ /* }}} */
-		vu16(xy) = vec_pack(vu32(xy), vu32(xy));
-		vu16(mests) = vec_pack(vu32(sads), vu32(sads));
-		vu16(mests) = vec_mergeh(vu16(mests), vu16(xy));
+		xy = vs8(vec_pack(vu32(xy), vu32(xy)));
+		mests = vu32(vec_pack(vu32(sads), vu32(sads)));
+		mests = vu32(vec_mergeh(vu16(mests), vu16(xy)));
 
 		vec_st(mests, 0, (unsigned int*)&vio.mests);
 	    } /* }}} */
@@ -605,7 +600,7 @@ int build_sub22_mests_altivec(BUILD_SUB22_MESTS_PDECL)
 #if ALTIVEC_TEST_FUNCTION(sub_mean_reduction)
     ALTIVEC_TEST_SUFFIX(sub_mean_reduction)(sub22set, reduction, &min_weight);
 #else
-    ALTIVEC_SUFFIX(sub_mean_reduction)(sub22set, reduction, &min_weight);
+    sub_mean_reduction_altivec(sub22set, reduction, &min_weight);
 #endif
     return sub22set->len;
 #endif
@@ -627,15 +622,15 @@ int build_sub22_mests_altivec_verify(BUILD_SUB22_MESTS_PDECL)
   len1 = _build_sub22_mests_altivec(BUILD_SUB22_MESTS_ARGS, 1 /*verify*/);
   for (checksum1 = i = 0; i < len1; i++) {
     checksum1 += sub22set->mests[i].weight;
-    checksum1 += intabs(sub22set->mests[i].x);
-    checksum1 += intabs(sub22set->mests[i].y);
+    checksum1 += abs(sub22set->mests[i].x);
+    checksum1 += abs(sub22set->mests[i].y);
   }
 
   len2 = ALTIVEC_TEST_WITH(build_sub22_mests)(BUILD_SUB22_MESTS_ARGS);
   for (checksum2 = i = 0; i < len2; i++) {
     checksum2 += sub22set->mests[i].weight;
-    checksum2 += intabs(sub22set->mests[i].x);
-    checksum2 += intabs(sub22set->mests[i].y);
+    checksum2 += abs(sub22set->mests[i].x);
+    checksum2 += abs(sub22set->mests[i].y);
   }
 
   if (len1 != len2 || checksum1 != checksum2) {
