@@ -223,43 +223,43 @@ void idct_altivec(IDCT_PDECL)
 #define MERGE_S16(hl,a,b) vec_merge##hl(vs16(a), vs16(b))
 
     bp = (vector signed short*)block;
-    vs16(x0) = vec_ld(0,    bp);
-    vs16(x4) = vec_ld(16*4, bp);
-    vs16(b00) = MERGE_S16(h, x0, x4);
-    vs16(b10) = MERGE_S16(l, x0, x4);
+    x0 = vfp(vec_ld(0,    bp));
+    x4 = vfp(vec_ld(16*4, bp));
+    b00 = vfp(MERGE_S16(h, x0, x4));
+    b10 = vfp(MERGE_S16(l, x0, x4));
     bp++;
-    vs16(x1) = vec_ld(0,    bp);
-    vs16(x5) = vec_ld(16*4, bp);
-    vs16(b20) = MERGE_S16(h, x1, x5);
-    vs16(b30) = MERGE_S16(l, x1, x5);
+    x1 = vfp(vec_ld(0,    bp));
+    x5 = vfp(vec_ld(16*4, bp));
+    b20 = vfp(MERGE_S16(h, x1, x5));
+    b30 = vfp(MERGE_S16(l, x1, x5));
     bp++;
-    vs16(x2) = vec_ld(0,    bp);
-    vs16(x6) = vec_ld(16*4, bp);
-    vs16(b40) = MERGE_S16(h, x2, x6);
-    vs16(b50) = MERGE_S16(l, x2, x6);
+    x2 = vfp(vec_ld(0,    bp));
+    x6 = vfp(vec_ld(16*4, bp));
+    b40 = vfp(MERGE_S16(h, x2, x6));
+    b50 = vfp(MERGE_S16(l, x2, x6));
     bp++;
-    vs16(x3) = vec_ld(0,    bp);
-    vs16(x7) = vec_ld(16*4, bp);
-    vs16(b60) = MERGE_S16(h, x3, x7);
-    vs16(b70) = MERGE_S16(l, x3, x7);
+    x3 = vfp(vec_ld(0,    bp));
+    x7 = vfp(vec_ld(16*4, bp));
+    b60 = vfp(MERGE_S16(h, x3, x7));
+    b70 = vfp(MERGE_S16(l, x3, x7));
 
-    vs16(b01) = MERGE_S16(h, b00, b40);
-    vs16(b11) = MERGE_S16(l, b00, b40);
-    vs16(b21) = MERGE_S16(h, b10, b50);
-    vs16(b31) = MERGE_S16(l, b10, b50);
-    vs16(b41) = MERGE_S16(h, b20, b60);
-    vs16(b51) = MERGE_S16(l, b20, b60);
-    vs16(b61) = MERGE_S16(h, b30, b70);
-    vs16(b71) = MERGE_S16(l, b30, b70);
+    b01 = vfp(MERGE_S16(h, b00, b40));
+    b11 = vfp(MERGE_S16(l, b00, b40));
+    b21 = vfp(MERGE_S16(h, b10, b50));
+    b31 = vfp(MERGE_S16(l, b10, b50));
+    b41 = vfp(MERGE_S16(h, b20, b60));
+    b51 = vfp(MERGE_S16(l, b20, b60));
+    b61 = vfp(MERGE_S16(h, b30, b70));
+    b71 = vfp(MERGE_S16(l, b30, b70));
 
-    vs16(x0) = MERGE_S16(h, b01, b41);
-    vs16(x1) = MERGE_S16(l, b01, b41);
-    vs16(x2) = MERGE_S16(h, b11, b51);
-    vs16(x3) = MERGE_S16(l, b11, b51);
-    vs16(x4) = MERGE_S16(h, b21, b61);
-    vs16(x5) = MERGE_S16(l, b21, b61);
-    vs16(x6) = MERGE_S16(h, b31, b71);
-    vs16(x7) = MERGE_S16(l, b31, b71);
+    x0 = vfp(MERGE_S16(h, b01, b41));
+    x1 = vfp(MERGE_S16(l, b01, b41));
+    x2 = vfp(MERGE_S16(h, b11, b51));
+    x3 = vfp(MERGE_S16(l, b11, b51));
+    x4 = vfp(MERGE_S16(h, b21, b61));
+    x5 = vfp(MERGE_S16(l, b21, b61));
+    x6 = vfp(MERGE_S16(h, b31, b71));
+    x7 = vfp(MERGE_S16(l, b31, b71));
 
 #undef MERGE_S16
     /* }}} */
@@ -267,10 +267,10 @@ void idct_altivec(IDCT_PDECL)
 
     /* convert to float {{{ */
 #define CTF(n) \
-    vs32(b##n##0) = vec_unpackh(vs16(x##n)); \
-    vs32(b##n##1) = vec_unpackl(vs16(x##n)); \
-    b##n##0 = vec_ctf(vs32(b##n##0), 0); \
-    b##n##1 = vec_ctf(vs32(b##n##1), 0); \
+    b##n##0 = vfp(vec_unpackh(vs16(x##n))); \
+    b##n##1 = vfp(vec_unpackl(vs16(x##n))); \
+    b##n##0 = vfp(vec_ctf(vs32(b##n##0), 0)); \
+    b##n##1 = vfp(vec_ctf(vs32(b##n##1), 0)); \
 
     CTF(0);
     CTF(1);
@@ -361,20 +361,20 @@ void idct_altivec(IDCT_PDECL)
     /* round, convert back to short and clip {{{ */
 
     /* cnsts0 = max = 255 = 0x00ff, cnsts2 = min = -256 = 0xff00 {{{ */
-    vu8(cnsts0) = vec_splat_u8(0);
-    vu8(x8) = vec_splat_u8(-1);
-    vu8(cnsts2) = vec_mergeh(vu8(x8), vu8(cnsts0));
-    vu8(cnsts0) = vec_mergeh(vu8(cnsts0), vu8(x8));
+    cnsts0 = vfp(vec_splat_u8(0));
+    x8 = vfp(vec_splat_u8(-1));
+    cnsts2 = vfp(vec_mergeh(vu8(x8), vu8(cnsts0)));
+    cnsts0 = vfp(vec_mergeh(vu8(cnsts0), vu8(x8)));
     /* }}} */
 
 #define CTS(n) \
-    b##n##0 = vec_round(b##n##0); \
-    b##n##1 = vec_round(b##n##1); \
-    vs32(b##n##0) = vec_cts(b##n##0, 0); \
-    vs32(b##n##1) = vec_cts(b##n##1, 0); \
-    vs16(b##n##0) = vec_pack(vs32(b##n##0), vs32(b##n##1)); \
-    vs16(b##n##0) = vec_min(vs16(b##n##0), vs16(cnsts0)); \
-    vs16(b##n##0) = vec_max(vs16(b##n##0), vs16(cnsts2)); \
+    b##n##0 = vfp(vec_round(b##n##0)); \
+    b##n##1 = vfp(vec_round(b##n##1)); \
+    b##n##0 = vfp(vec_cts(b##n##0, 0)); \
+    b##n##1 = vfp(vec_cts(b##n##1, 0)); \
+    b##n##0 = vfp(vec_pack(vs32(b##n##0), vs32(b##n##1))); \
+    b##n##0 = vfp(vec_min(vs16(b##n##0), vs16(cnsts0))); \
+    b##n##0 = vfp(vec_max(vs16(b##n##0), vs16(cnsts2))); \
     vec_st(vs16(b##n##0), 0, bp);
 
 
