@@ -46,6 +46,16 @@ extern "C" {
 #if	!defined(HAVE_FMAX)
 	double fmax(double, double);
 #endif
+#if	!defined(HAVE_FMIN)
+	double fmin(double, double);
+#endif
+
+/* take a pointer, and force it to be aligned on a particular boundary */
+#define ALIGN_PTR(p,a) ((void *)( (((size_t)(p))+(a)-1)&~((size_t)(a)-1)))
+
+extern const char *disable_simd_flags[];
+extern int simd_name_ok(char *);
+extern int disable_simd(char *);
 
 #ifdef __cplusplus
 }
