@@ -1,5 +1,5 @@
 /* 
- * $Id: y4munsharp.c,v 1.10 2007/04/01 17:21:57 sms00 Exp $
+ * $Id: y4munsharp.c,v 1.11 2009-09-19 19:01:43 sms00 Exp $
  *
  * Constructed using:
  * unsharp.c 0.10 -- This is a plug-in for the GIMP 1.0
@@ -46,9 +46,7 @@
 #define MAX(a,b) ((a) >= (b) ? (a) : (b))
 #define ROUND(x) ((int) ((x) + 0.5))
 
-extern char *__progname;
-
-void usage(void);
+void usage(char *);
 void y4munsharp(void);
 static void get_column(u_char *, u_char *, int, int);
 static void put_column(u_char *, u_char *, int, int);
@@ -96,7 +94,7 @@ main(int argc, char **argv)
 				if	(i != 3)
 					{
 					mjpeg_error("-L r,a,t");
-					usage();
+					usage(argv[0]);
 					}
 				break;
 			case	'C':
@@ -105,7 +103,7 @@ main(int argc, char **argv)
 				if	(i != 3)
 					{
 					mjpeg_error("-C r,a,t");
-					usage();
+					usage(argv[0]);
 					}
 				break;
 			case	'v':
@@ -115,7 +113,7 @@ main(int argc, char **argv)
 				break;
 			case	'h':
 			default:
-				usage();
+				usage(argv[0]);
 				break;
 			}
 		}
@@ -446,7 +444,7 @@ static void
 blur_line (double *ctable, double *cmatrix, int cmatrix_length,
 	   u_char  *cur_col, u_char  *dest_col, int y)
   {
-  double scale, sum, *cmatrix_p, *ctable_p;
+  double scale, sum, *ctable_p;
   int i=0, j=0, row, cmatrix_middle = cmatrix_length/2;
   u_char  *cur_col_p, *cur_col_p1, *dest_col_p;
 
@@ -505,7 +503,6 @@ blur_line (double *ctable, double *cmatrix, int cmatrix_length,
 	   for  (i = 0; i < 1; i++)
 	        {
 	        sum = 0;
-	        cmatrix_p = cmatrix;
 	        cur_col_p1 = cur_col_p;
 	        ctable_p = ctable;
 	        for  (j = cmatrix_length; j>0; j--)
@@ -544,7 +541,7 @@ blur_line (double *ctable, double *cmatrix, int cmatrix_length,
 static int
 gen_convolve_matrix(double radius, double **cmatrix_p)
 	{
-	int matrix_length, matrix_midpoint, i, j;
+	int matrix_length, i, j;
 	double *cmatrix, std_dev, sum, base_x;
 	
   /* we want to generate a matrix that goes out a certain radius
@@ -565,7 +562,6 @@ gen_convolve_matrix(double radius, double **cmatrix_p)
 	/* go out 'radius' in each direction */
 	matrix_length = 2 * ceil(radius-0.5) + 1;
 	if (matrix_length <= 0) matrix_length = 1;
-	matrix_midpoint = matrix_length/2 + 1;
 	*cmatrix_p = (double *)malloc(sizeof (double) * matrix_length);
 	cmatrix = *cmatrix_p;
 
@@ -630,18 +626,18 @@ gen_lookup_table(double *cmatrix, int cmatrix_length)
 	return(lookup_table);
 	}
 
-void usage(void)
+void usage(char *pgm)
 	{
-	fprintf(stderr, "%s: usage: [-v 0|1|2] [-N] [-L radius,amount,threshold] [-C radius,amount,threshold]\n", __progname);
+	fprintf(stderr, "%s: usage: [-v 0|1|2] [-N] [-L radius,amount,threshold] [-C radius,amount,threshold]\n", pgm);
 	fprintf(stderr, "%s:\tradius and amount are floating point numbers\n",
-		__progname);
-	fprintf(stderr, "%s:\tthreshold is integer.\n", __progname);
-	fprintf(stderr, "%s:\tdefault for -L is 2.0,0.3,4\n", __progname);
+		pgm);
+	fprintf(stderr, "%s:\tthreshold is integer.\n", pgm);
+	fprintf(stderr, "%s:\tdefault for -L is 2.0,0.3,4\n", pgm);
 	fprintf(stderr, "%s:\tchroma not filtered UNLESS -C used, no default\n",
-		__progname);
-	fprintf(stderr, "%s:-v verbose 0=quiet 1=normal 2=debug (default: 1)\n", __progname);
-	fprintf(stderr, "%s:-N disables the Y' 16-235 clip/core\n", __progname);
-	fprintf(stderr, "%s:   disables the CbCr 16-240 clip/core\n", __progname);
-	fprintf(stderr, "%s:   (allow the entire 0 to 255 range).\n", __progname);
+		pgm);
+	fprintf(stderr, "%s:-v verbose 0=quiet 1=normal 2=debug (default: 1)\n", pgm);
+	fprintf(stderr, "%s:-N disables the Y' 16-235 clip/core\n", pgm);
+	fprintf(stderr, "%s:   disables the CbCr 16-240 clip/core\n", pgm);
+	fprintf(stderr, "%s:   (allow the entire 0 to 255 range).\n", pgm);
 	exit(1);
 	}
