@@ -312,7 +312,7 @@ temporal_filter_planes_MC (int idx, int w, int h, int t)
 		v /= 16;
 
 		// add weighted and translated but non-filtered test-pixel
-		d = t - abs (r-v);
+		d = t - r-v;
 		d = d<0? 0:d;
 	  	c += d;
           	m += *(f1+(x+sx+x1)+(y+sy+y1)*w)*d;
@@ -334,7 +334,7 @@ temporal_filter_planes_MC (int idx, int w, int h, int t)
 		v /= 16;
 
 		// add weighted and translated but non-filtered test-pixel
-		d = t - abs (r-v);
+		d = t - r-v;
 		d = d<0? 0:d;
 	  	c += d;
           	m += *(f2+(x+sx+x2)+(y+sy+y2)*w)*d;
@@ -356,7 +356,7 @@ temporal_filter_planes_MC (int idx, int w, int h, int t)
 		v /= 16;
 
 		// add weighted and translated but non-filtered test-pixel
-		d = t - abs (r-v);
+		d = t - r-v;
 		d = d<0? 0:d;
 	  	c += d;
           	m += *(f3+(x+sx+x3)+(y+sy+y3)*w)*d;
@@ -378,7 +378,7 @@ temporal_filter_planes_MC (int idx, int w, int h, int t)
 		v /= 16;
 
 		// add weighted and translated but non-filtered test-pixel
-		d = t - abs (r-v);
+		d = t - r-v;
 		d = d<0? 0:d;
 	  	c += d;
           	m += *(f5+(x+sx+x5)+(y+sy+y5)*w)*d;
@@ -400,7 +400,7 @@ temporal_filter_planes_MC (int idx, int w, int h, int t)
 		v /= 16;
 
 		// add weighted and translated but non-filtered test-pixel
-		d = t - abs (r-v);
+		d = t - r-v;
 		d = d<0? 0:d;
 	  	c += d;
           	m += *(f6+(x+sx+x6)+(y+sy+y6)*w)*d;
@@ -422,7 +422,7 @@ temporal_filter_planes_MC (int idx, int w, int h, int t)
 		v /= 16;
 
 		// add weighted and translated but non-filtered test-pixel
-		d = t - abs (r-v);
+		d = t - r-v;
 		d = d<0? 0:d;
 	  	c += d;
           	m += *(f7+(x+sx+x7)+(y+sy+y7)*w)*d;
@@ -708,7 +708,7 @@ void temporal_filter_planes_p (int idx, int w, int h, int t)
 		d += *(f3+1+w);
 		d /= 16;
 
-		d = t - abs (r-d);
+		d = t - r - d;
 		d = d<0? 0:d;
 		c += d;
 		m += *(f3)*d*2;
@@ -724,7 +724,7 @@ void temporal_filter_planes_p (int idx, int w, int h, int t)
 		d += *(f2+1+w);
 		d /= 16;
 
-		d = t - abs (r-d);
+		d = t - r-d;
 		d = d<0? 0:d;
 		c += d;
 		m += *(f2)*d*2;
@@ -740,7 +740,7 @@ void temporal_filter_planes_p (int idx, int w, int h, int t)
 		d += *(f1+1+w);
 		d /= 16;
 
-		d = t - abs (r-d);
+		d = t - r-d;
 		d = d<0? 0:d;
 		c += d;
 		m += *(f1)*d*2;
@@ -756,7 +756,7 @@ void temporal_filter_planes_p (int idx, int w, int h, int t)
 		d += *(f5+1+w);
 		d /= 16;
 
-		d = t - abs (r-d);
+		d = t - r-d;
 		d = d<0? 0:d;
 		c += d;
 		m += *(f5)*d*2;
@@ -772,7 +772,7 @@ void temporal_filter_planes_p (int idx, int w, int h, int t)
 		d += *(f6+1+w);
 		d /= 16;
 
-		d = t - abs (r-d);
+		d = t - r-d;
 		d = d<0? 0:d;
 		c += d;
 		m += *(f6)*d*2;
@@ -788,7 +788,7 @@ void temporal_filter_planes_p (int idx, int w, int h, int t)
 		d += *(f7+1+w);
 		d /= 16;
 
-		d = t - abs (r-d);
+		d = t - r-d;
 		d = d<0? 0:d;
 		c += d;
 		m += *(f7)*d*2;
@@ -808,22 +808,25 @@ void temporal_filter_planes_p (int idx, int w, int h, int t)
 
 #if defined(__SSE2__)
 /* 4 to 5 times faster */
-void filter_plane_median_sse2(uint8_t *plane, int w, int h, int level) {
+void filter_plane_median_sse2(uint8_t *plane, int w, int h, int level)
+	{
 	int i;
-	int avg; /*should not be needed any more */
-	int cnt; /* should not be needed any more */
 	uint8_t * p;
 	uint8_t * d;
-	
-	if(level==0) return;
-	
+#if !defined(__SSE3__)
+	int	avg, cnt;
+#endif
+
+	if (level==0)
+		return;
 	p = plane;
 	d = scratchplane1;
 
 	// remove strong outliers from the image. An outlier is a pixel which lies outside
 	// of max-thres and min+thres of the surrounding pixels. This should not cause blurring
 	// and it should leave an evenly spread noise-floor to the image.
-	for (i=0; i<=(w*h); i+=14) {
+	for (i=0; i<=(w*h); i+=14)
+		{
 		__m128i t, c, b, min, max, minmin, maxmax;
 		
 		t = _mm_loadu_si128((__m128i *)&p[i-1-w]);
@@ -846,7 +849,7 @@ void filter_plane_median_sse2(uint8_t *plane, int w, int h, int level) {
 		/* write 14 valid pixels, the 2 remaining bytes are overwritten subsequently
 		 * or lie outside the frame area */
 		_mm_storeu_si128((__m128i *)&d[i], c);
-	}
+		}
 	
 	// in the second stage we try to average similar spatial pixels, only. This, like
 	// a median, should also not reduce sharpness but flatten the noisefloor. This
@@ -872,7 +875,7 @@ void filter_plane_median_sse2(uint8_t *plane, int w, int h, int level) {
 #endif
 	
 	for (i=0; i<=(w*h); i+=4)
-	{
+		{
 		uint64_t k0, k1, k2, k3, k6;
 		__m128i c0, c1, v[4], t[4], e[4], a[4];
 		
@@ -994,6 +997,7 @@ void filter_plane_median_sse2(uint8_t *plane, int w, int h, int level) {
 		int j;
 		__m128 f0, f1, f2, flvl;
 		__m128i zero, vv;
+
 		flvl = _mm_set1_ps((float)level);
 		zero = _mm_setzero_si128();
 		vv = _mm_set_epi32((k6 >> 40) & 0xff, (k6 >> 32) & 0xff, (k6 >> 24) & 0xff, (k6 >> 16) & 0xff);
@@ -1073,11 +1077,11 @@ void filter_plane_median_sse2(uint8_t *plane, int w, int h, int level) {
 		
 		d += 4;
 		p += 4;
-	}
+		}
 	_mm_empty();
 	
 	memcpy(plane,scratchplane2,w*h);
-}
+	}
 #endif
 
 void filter_plane_median_p ( uint8_t * plane, int w, int h, int level)
