@@ -26,8 +26,8 @@ YfFrame_t *
 YfInitFrame(YfFrame_t *frame, const YfTaskCore_t *h0)
 {
   if (!frame) {
-    if (!(frame = malloc(FRAMEBYTES(h0->width, h0->height)))) {
-      perror("malloc");
+    if (!(frame = malloc(FRAMEBYTES(y4m_si_get_chroma(&h0->si), h0->width, h0->height)))) {
+      mjpeg_error("malloc");
       return NULL;
     }
   }
