@@ -34,6 +34,8 @@
 #define DEFAULTNOISE 10
 #define DEFAULTDEINTL 10
 
+int verbose = 1;
+
 static void
 buf_debug(char *buf, FILE *fp, const char *format, ...)
 {
@@ -46,12 +48,8 @@ buf_debug(char *buf, FILE *fp, const char *format, ...)
     if (fp) {
       fputs(buf, fp);
     } else {
-#ifdef MJPEGTOOLS
       buf[n] = '\0';
-      mjpeg_debug(buf);
-#else
-      fputs(buf, stderr);
-#endif
+      mjpeg_debug("%s", buf);
     }
     buf[0] = '\0';
   }
@@ -92,13 +90,16 @@ typedef struct {
   } framestat[0];
 } YfTask_t;
 
-DEFINE_STD_YFTASKCLASS(yuvkineco);
-DECLARE_YFTASKCLASS(yuvycsnoise);
+static  YfTaskCore_t *do_init(int, char **, const YfTaskCore_t *);
+static  void do_fini(YfTaskCore_t *);
+static  int do_frame(YfTaskCore_t *, const YfTaskCore_t *, const YfFrame_t *);
+
+const   YfTaskClass_t yuvkineco = { do_init, do_fini, do_frame };
 
 static const char *
 do_usage(void)
 {
-  return "[-{u|s}] [-F OutputFPSCODE] [-S YCSNoiseThreashold] [-n NoiseLevel] "
+  return "[-{u|s}] [-F OutputFPSCODE] [-n NoiseLevel] "
     "[-c CycleSearchThreashold] [-i DeinterlacePixelsPermil] "
     "{[-C OutputCycleListName] | -[ON] InputCycleListName}";
 }
@@ -112,15 +113,13 @@ do_init(int argc, char **argv, const YfTaskCore_t *h0)
   int c;
   int cytype = 0;
   char *cyname = NULL;
-  char *ycsthres = NULL;
-  YfTaskCore_t *hycs = NULL;
   int dthr16 = DEFAULTDTHR16;
   int noiselevel = DEFAULTNOISE;
   int deintlval = DEFAULTDEINTL;
   int output_interlace = 0;
 
   fpscode = h0->fpscode;
-  while ((c = getopt(argc, argv, "usF:S:n:c:i:C:O:N:")) != -1) {
+  while ((c = getopt(argc, argv, "usF:n:c:i:C:O:N:")) != -1) {
     switch (c) {
     case 'u':
       output_interlace = 1;
@@ -131,9 +130,6 @@ do_init(int argc, char **argv, const YfTaskCore_t *h0)
     case 'F':
       fpscode = atoi(optarg);
       break;
-    case 'S':
-      ycsthres = optarg;
-      break;
     case 'n':
       noiselevel = atoi(optarg);
       break;
@@ -154,26 +150,17 @@ do_init(int argc, char **argv, const YfTaskCore_t *h0)
     }
   }
   if (cytype != 'N' && (h0->fpscode < 3 || 5 < h0->fpscode)) {
-    WERROR("unsupported input fps");
+    mjpeg_error("unsupported input fps");
     return NULL;
   }
   if (fpscode < 1 || h0->fpscode < fpscode) {
-    WERROR("illeagal output fpscode");
+    mjpeg_error("illegal output fpscode");
     return NULL;
   }
   if (y4m_si_get_interlace(&h0->si) == Y4M_ILACE_BOTTOM_FIRST) {
-    WERROR("unsupported field order");
+    mjpeg_error("unsupported field order");
     return NULL;
   }
-  if (ycsthres) {
-    char ycssopt[] = "-S";
-    char *ycsargv[] = { argv[0], ycssopt, ycsthres, NULL, };
-    optind = 1;
-    if (!(hycs = YfAddNewTask(&yuvycsnoise,
-			   (sizeof ycsargv / sizeof ycsargv[0]) - 1,
-			   ycsargv, h0)))
-      return NULL;
-  }
   nframes = ((cytype && cytype != 'C')? 2:
 	     (h0->fpscode == 3)? 49: 9);
   nfields = ((y4m_si_get_interlace(&h0->si) == Y4M_UNKNOWN)?
@@ -196,7 +183,7 @@ do_init(int argc, char **argv, const YfTaskCore_t *h0)
 		    ((((!cytype || cytype == 'C')? (sizeof h->framestat[0]):
 		       0) + FRAMEBYTES(y4m_si_get_chroma(&h0->si), h0->width, h0->height)) * nframes) +
 		    ((0 <= deintlval)? FRAMEBYTES(y4m_si_get_chroma(&h0->si), h0->width, h0->height): 0)),
-		   (hycs? hycs: h0));
+		   h0);
   if (!h)
     return NULL;
   if (output_interlace == 2 &&
@@ -227,12 +214,12 @@ do_init(int argc, char **argv, const YfTaskCore_t *h0)
   if (!cytype)
     goto RETURN;
   if ((h->cyfp = fopen(cyname, ((cytype == 'C')? "w": "r"))) == NULL) {
-    perror(cyname);
+    mjpeg_error("fopen(%s): %s", cyname, strerror(errno));
     goto ERROR_RETURN;
   }
   if (cytype == 'C') {
     fprintf(h->cyfp, "# 2-3 pull down cycle list\n"
-"# Generated by 'yuvkineco%s -F%d%s%s -n%d -c%d -i%d -C %s'\n"
+"# Generated by 'yuvkineco%s -F%d -n%d -c%d -i%d -C %s'\n"
 "#\n"
 "# Each character in list mean how each frame processed:\n"
 "#   O,o:    output\n"
@@ -277,10 +264,10 @@ do_init(int argc, char **argv, const YfTaskCore_t *h0)
 "#\n"
 "SIZE:%dx%d OLD_FPS:%d NEW_FPS:%d # DON'T CHANGE THIS LINE!!!",
 	    ((output_interlace == 1)? " -u": (2 <= output_interlace)? " -s": ""), fpscode,
-	    (ycsthres? " -S": ""), (ycsthres? ycsthres: ""),
 	    noiselevel, dthr16, deintlval,
 	    cyname, cyname, cyname, cyname, cyname,
 	    h0->width, h0->height, h0->fpscode, fpscode);
+    fflush(h->cyfp);
   } else {
     int width, height, oldfps, newfps;
     char buff[128];
@@ -292,24 +279,24 @@ do_init(int argc, char **argv, const YfTaskCore_t *h0)
 		 &width, &height, &oldfps, &newfps) == 4) {
 	if (width != h0->width || height != h0->height ||
 	    ((h->cytype == 'O')? oldfps: newfps) != h0->fpscode) {
-	  WERROR("input and cycle list not match.");
+	  mjpeg_error("input and cycle list not match.");
 	  goto ERROR;
 	}
 	if (h->cytype == 'O' && newfps < fpscode) {
-	  WERROR("output fpscode greater than specified at 1st try.");
+	  mjpeg_error("output fpscode greater than specified at 1st try.");
 	  goto ERROR;
 	}
 	goto RETURN;
       }
     }
-    WERROR("broken cycle list.");
+    mjpeg_error("broken cycle list.");
   ERROR:
     fclose(h->cyfp);
     goto ERROR_RETURN;
   }
- RETURN:
+RETURN:
   return (YfTaskCore_t *)h;
- ERROR_RETURN:
+ERROR_RETURN:
   if (0 <= h->deintl)
     h->nframes++;
   while (0 <= --h->nframes)
@@ -432,20 +419,9 @@ deinterlace(YfTask_t *h, unsigned char *data, int width, int height, int noise)
 	  n++;
 	else if (!(ynow < ymin || ymax < ynow))
 	  continue;
-#if 0
-	if (i < height - 3) {
-	  int ynx2 = data[((i + 3) * width) + j];
-	  if ((ynx2 < ynxt && ynxt < ymin) || (ymax < ynxt && ynxt < ynx2))
-	    continue;
-	}
-#endif
 	ynow = (ymin + ymax) / 2;
 	data[((i    ) * width) + j] = ynow;
 	if (i < height - 3) {
-#if 0
-	  int ynx2 = data[((i + 4) * width) + j];
-	  if ((ynx2 < ymin && ynxt < ymin) || (ymax < ynx2 && ymax < ynxt))
-#endif
 	    continue;
 	}
 	data[((i + 2) * width) + j] = ynow;
@@ -527,6 +503,7 @@ do_frame(YfTaskCore_t *handle, const YfTaskCore_t *h0, const YfFrame_t *frame0)
       unsigned int noise;
       int ipre = (h->iget - 1) % h->nframes;
       YfFrame_t *fpre = (YfFrame_t *)((char *)h->frame + (ipre * framebytes));
+
       memset(&h->framestat[b], 0, sizeof h->framestat[b]);
       for (i = h->nfields; i < h->_.height; i += h->nfields) {
 	for (j = 0; j < h->_.width; j++) {
@@ -537,7 +514,7 @@ do_frame(YfTaskCore_t *handle, const YfTaskCore_t *h0, const YfFrame_t *frame0)
 	    noise = yget;
 	  else if (256 - yget < noise)
 	    noise = 256 - yget;
-	  if ((d = ypre - yget) < 0) d = -d;
+	  d = abs(ypre - yget);
 	  h->framestat[b].diffdist[(d < NOISEMAX)? d: NOISEMAX]++;
 	  if (noise < d) {
 	    d -= noise;
@@ -547,29 +524,29 @@ do_frame(YfTaskCore_t *handle, const YfTaskCore_t *h0, const YfFrame_t *frame0)
 	    unsigned int ygte;
 	    ypre = fpre->data[((i - 1) * h->_.width) + j];
 	    ygte = fget->data[((i - 1) * h->_.width) + j];
-	    if ((d = yget - ygte) < 0) d = -d;
+	    d = abs(yget - ygte);
 	    if (noise < d) {
 	      d -= noise;
 	      h->framestat[b].eoediff -= (((d * d) + 16) >> 5);
 	    }
-	    if ((d = yget - ypre) < 0) d = -d;
+	    d = abs(yget - ypre);
 	    if (noise < d) {
 	      d -= noise;
 	      h->framestat[b].eoediff += (((d * d) + 16) >> 5);
 	    }
 	    ypre = fpre->data[((i + 1) * h->_.width) + j];
 	    ygte = fget->data[((i + 1) * h->_.width) + j];
-	    if ((d = yget - ygte) < 0) d = -d;
+	    d = abs(yget - ygte);
 	    if (noise < d) {
 	      d -= noise;
 	      h->framestat[b].eoediff -= (((d * d) + 16) >> 5);
 	    }
-	    if ((d = yget - ypre) < 0) d = -d;
+	    d = abs(yget - ypre);
 	    if (noise < d) {
 	      d -= noise;
 	      h->framestat[b].eoediff += (((d * d) + 16) >> 5);
 	    }
-	    if ((d = ypre - ygte) < 0) d = -d;
+	    d = abs(ypre - ygte);
 	    if (noise < d) {
 	      d -= noise;
 	      h->framestat[b].ediff += (((d * d) + 8) >> 4);
@@ -641,31 +618,8 @@ do_frame(YfTaskCore_t *handle, const YfTaskCore_t *h0, const YfFrame_t *frame0)
 	  notdrop[i] |= 1;
       if (1 < h->nfields) {	/* check field merged */
 	int merged = 0;
-#if 0
-	/* codes for video editing fade-in/out */
-	long dmax = -0x7fffffffL, d2nd = -0x7fffffffL, d3rd = -0x7fffffffL;
-	for (i = 1; i < (h->nframes / 2) + 2 && i < h->iget - h->iuse; i++) {
-	  long eoediff = h->framestat[(b + i) % h->nframes].eoediff;
-	  if (eoediff == -9999999)
-	    continue;
-	  if        (dmax < eoediff) {
-	    d3rd = d2nd; d2nd = dmax; dmax = eoediff;
-	  } else if (d2nd < eoediff) {
-	    d3rd = d2nd; d2nd = eoediff;
-	  } else if (d3rd < eoediff) {
-	    d3rd = eoediff;
-	  }
-	}
-	if (d3rd == -0x7fffffffL)
-	  d3rd = 0;
-	if (1 < verbose)
-	  buf_debug(debugbuf, NULL, "%6ld", d3rd);
-	for (i = 1; i < (h->nframes / 2) + 2 && i < h->iget - h->iuse; i++)
-	  if (h->framestat[(b + i) % h->nframes].eoediff < d3rd - (long)dthr)
-#else
 	    for (i = 1; i < h->iget - h->iuse - 1; i++)
 	      if (h->framestat[(b + i) % h->nframes].eoediff < -(long)dthr)
-#endif
 		{
 		  merged = 1;
 		  notdrop[i - 1] &= ~2;
@@ -700,9 +654,7 @@ do_frame(YfTaskCore_t *handle, const YfTaskCore_t *h0, const YfFrame_t *frame0)
       if (idrp == h->nframes / 2) {
 	unsigned long noisetotal;
 	int bdrp;
-#if 0
-	int diff1024;
-#endif
+
 	for (i = 0; i < idrp; i++)
 	  if (notdrop[i] == '0')
 	    goto DONE;
@@ -722,27 +674,7 @@ do_frame(YfTaskCore_t *handle, const YfTaskCore_t *h0, const YfFrame_t *frame0)
 	    break;
 	  noisetotal -= h->u.noise.dist[i];
 	}
-#if 0
-	diff1024 = (i - h->u.noise.level) * 1024;
-#endif
 	h->u.noise.level = i;
-#if 0
-	if (diff1024) {
-	  for (i = 0; i < h->nframes; i++) {
-	    if (diff1024 < -(int)h->framestat[i].odiff)
-	      h->framestat[i].odiff = 0;
-	    else
-	      h->framestat[i].odiff += diff1024;
-	  }
-	  if (1 < h->nfields)
-	    for (i = 0; i < h->nframes; i++) {
-	      if (diff1024 < -(int)h->framestat[i].ediff)
-		h->framestat[i].ediff = 0;
-	      else
-		h->framestat[i].ediff += diff1024;
-	    }
-	}
-#endif
       }
     DONE:
       if (1 < verbose)
@@ -825,7 +757,7 @@ do_frame(YfTaskCore_t *handle, const YfTaskCore_t *h0, const YfFrame_t *frame0)
 	  break;
 	if (isalpha(c) || (c == '_' && h->cytype == 'O')) {
 	  if (h->u.cy.buff + 30 <= d) {
-	    WWARN("too long line in cycle list.");
+	    mjpeg_warn("too long line in cycle list.");
 	    break;
 	  }
 	  *d++ = c;
@@ -899,3 +831,46 @@ do_frame(YfTaskCore_t *handle, const YfTaskCore_t *h0, const YfFrame_t *frame0)
   }
   return 0;
 }
+extern const YfTaskClass_t yuvstdin;
+extern const YfTaskClass_t yuvstdout;
+extern const YfTaskClass_t yuvkineco;
+
+int
+main(int argc, char **argv)
+{
+  YfTaskCore_t *h, *hreader;
+  int ret;
+  char *p;
+
+  if (1 < argc && (!strcmp(argv[1], "-?") ||
+		   !strcmp(argv[1], "-h") ||
+		   !strcmp(argv[1], "--help"))) {
+    mjpeg_error_exit1("Usage: %s", do_usage());
+  }
+  if ((p = getenv("MJPEG_VERBOSITY")))
+    verbose = atoi(p);
+
+   y4m_accept_extensions(1);
+
+  ret = 1;
+  if (!(hreader = YfAddNewTask(&yuvstdin, argc, argv, NULL)))
+    goto RETURN;
+  if (!YfAddNewTask(&yuvkineco, argc, argv, hreader))
+    goto FINI;
+  if (!YfAddNewTask(&yuvstdout, argc, argv, hreader))
+    goto FINI;
+
+  ret = (*yuvstdin.frame)(hreader, NULL, NULL);
+  if (ret == Y4M_ERR_EOF)
+    ret = Y4M_OK;
+  if (ret != Y4M_OK)
+    mjpeg_error("%s", y4m_strerr(ret));
+
+ FINI:
+  for (h = hreader; h; h = hreader) {
+    hreader = h->handle_outgoing;
+    (*h->method->fini)(h);
+  }
+ RETURN:
+  return ret;
+}
